-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------

-----------------------------------------------------------
-- Award Functions
-- These are the functions that allocate the DKP to one or many as needed
--
RDKP_AwardPlugins = {};

function RDKP_AwardPluginInit()
	-- A function specifically handling the drop down list of the available plugins.
	RDKP_AwardPluginKhaosSettingsSetup();
	RDKP_AwardPlugins[RDKP_Settings.PrioritySystem].KhaosSettings();
end

function RDKP_AwardPluginKhaosSettingsSetup()
	local tOptions = {};
	for k, v in pairs(RDKP_AwardPlugins) do
		tOptions[k] = k;
	end
	local KhaosOption =  {
		id = "PrioritySystem"; -- Unique within the Set
		text = "Priority System Plugin to Use:";
		difficulty = 1;
		helptext = "Select which priority system to use for determining priority for loot.";
		callback = function(state) 
				RDKP_Settings_Temp.PrioritySystem = state.value; 
			end;
		type = K_PULLDOWN;
		setup = {
			options = tOptions;
			multiSelect=false;
		};
		disabled = {
			value = "RATIO";
		};
		default = {
			value = "RATIO";
		};
	}
	return KhaosOption;
end

function RDKP_GetAwardPriveledges()
	-- Return true if this is the person who is the RL
	-- Only the RL/Officer In Charge should be doing this.
	-- Get Current Status in the raid
	-- Get Current Rank.
	-- figure that if player CanEditOfficerNote() then they are sufficient rank w/i the guild to be an officer w/ Raid point priv. 
	RDKP_Debug("GetAwardPriveledges");
	-- For right now, only the raid leader can post current Raid Awards.  
	-- This needs to be expanded for Guild-Wide updates.
	if RDKP_b_Debug then
		return true;
	elseif IsRaidLeader() and (RDKP_Table_RaidInfo["RecordRaid"]) then
		return true;
	end
	return nil;
end

function RDKP_AwardOnKill(szKill)
	RDKP_Debug("RDKP_AwardOnKill()")
	RDKP_PrintToDebugLog(szKill);
	local iStart, iEnd, szMobDied = string.find(szKill, RDKP_SZ_PATTERN_DEATH1);
	if ( szMobDied == nil ) then
		local iStart, iEnd, szMobDied = string.find(szKill, RDKP_SZ_PATTERN_DEATH2);
	end
	RDKP_Debug(szMobDied);
	-- Check if this happened in a zone we care about.
	if RDKP_Khaos_TrackThisZone(GetRealZoneText()) then
		-- Check string to pre-defined list of bosses
		if RDKP_AwardPlugins[RDKP_Settings.PrioritySystem] then 
			RDKP_AwardPlugins[RDKP_Settings.PrioritySystem].Kill(szMobDied, GetRealZoneText());
		else
			error("RDKP_AwardPlugins[" .. RDKP_Settings.PrioritySystem .. "] == nil", 1);
		end
	end
end

function RDKP_AwardSetupOnTime()
	-- Check to see if there is already a function scheduled
	local iWhen = 0;
	local iDKP = 0;
	local iDKPperTrans = 0;
	local szScheduleName = "";
	if not RDKP_Table_RaidPlayers then
		RDKP_UpdateRaidList();
	end
	iDKP = RDKP_Settings.DKP.TIME;
	-- Then call the function for the individual on that counter.
	if (iDKP) then 
		if iDKP < 12 then 
			-- Trying to reduce the number of transactions to no more than 1 every 5 mins.
			iWhen = 3600/iDKP;
			RDKP_Table_RaidInfo.iDKPperTrans = 1;
		else
			iWhen = 3600/12;
			RDKP_Table_RaidInfo.iDKPperTrans = iDKP/12;
		end
		if not (RDKP_Table_RaidInfo.iDKPperTrans) then
			errror ("Error: iDKPperTrans == " .. tostring(RDKP_Table_RaidInfo.iDKPperTrans), 1);
		else
			if (RDKP_AwardPlugins[RDKP_Settings.PrioritySystem]) then 
				RDKP_Debug("Chronos.scheduleRepeating(" .. szScheduleName .. ", " .. iWhen .. ", <functioncall>)");
				Chronos.scheduleRepeating("RDKP_DKPforTime", iWhen, (RDKP_AwardPlugins[RDKP_Settings.PrioritySystem].Time));
			else
				error ("RDKP_AwardPlugins[" .. RDKP_Settings.PrioritySystem .. "] == nil");
			end
		end
	end
end

function RDKP_AwardOnLoot(szLoot)
	-- Post kill, someone gets the loot.  
	-- If the loot is at a certain level, then the reciever needs to be credited for it
	-- Also need to track for shards here.
	-- If a shard is created w/ the loot that was just received, then the GR needs to be removed.
	-- Code stolen from CTRT
	if RDKP_GetAwardPriveledges() then
		RDKP_PrintToDebugLog(szLoot);
		if RDKP_Khaos_TrackThisZone(GetRealZoneText()) then
			local sPlayer, sLink;
			local iStart, iEnd, sPlayerName, sItem = string.find(szLoot, RDKP_SZ_PATTERN_LOOT1);
			if ( sPlayerName ) then
				sPlayer = sPlayerName;
				sLink = sItem;
			else
				local iStart, iEnd, sItem = string.find(szLoot, RDKP_SZ_PATTERN_LOOT2);
				if ( sItem ) then
					sPlayer = UnitName("player");
					sLink = sItem;
				end
			end
			-- Now that we have the gear, we need to get it's value
			if ( sLink and sPlayer ) then
				RDKP_Debug("sLink: " .. sLink)
				RDKP_Debug("sPlayer: " .. sPlayer)
				local iStart, iEnd, sColor, idItem, sName = string.find(sLink, RDKP_SZ_PATTERN_LOOT);
				if (sColor and sName) then
					RDKP_AwardPlugins[RDKP_Settings.PrioritySystem].Loot(sName, RDKP_MAP_STRINGS.RARITY[sColor], sPlayer, GetRealZoneText())
				else
					if not sColor then
						error("Looting: Color value not aquired from loot link.", 1)
					end
					if not sName then
						error("Looting: Name of item not aquired from loot link.", 1)
					end
				end
			end
		end
	end
end

function RDKP_Transaction_Adjustment(szPlayer, iDKP, iGR, szComment)
	-- This will be the single atomic operation that will be presented in the raid log.
	-- and communicated to the rest of the raid.
	-- szPlayer = who gets the amount
	-- iDKP = DKP amount change
	-- iGR = GR amount change
	-- These adjustments should be unique to themselves.
	if (szPlayer == nil) then
		-- No point in building a transaction if there's no player name.
		return nil;
	end
	if (not iDKP) then 
		iDKP = 0;
	end
	if (not iGR) then
		iGR = 0;
	end
	if ((iDKP == 0) and (iGR == 0)) then 
		-- No point in building a transaction that will result in no change.
		return nil;
	end
	if (szComment == nil) then 
		szComment = "Adjustment";
	end
	-- Check to see if we presently have a RAID ID.
	local szTransactionKey = RDKP_Build_RaidKeyID(); 
	if (not (RDKP_Table_RaidInfo["RDKP_RaidID"] == nil)) then
		szTransactionKey = RDKP_Table_RaidInfo["RDKP_RaidID"];
	end
	RDKP_Debug("iDKP: " .. iDKP .. "iGR: " ..  iGR);
	-- If transaction adjustments are being done during a raid, the transaction ID will need to be dynamicly generated.
	local iCurTransactionID = 1;
	if (not (RDKP_Table_RaidInfo.RDKP_TransactionID == nil)) then
		RDKP_Table_RaidInfo.RDKP_TransactionID = RDKP_Table_RaidInfo.RDKP_TransactionID + 1;
		iCurTransactionID = RDKP_Table_RaidInfo.RDKP_TransactionID;
	end
	return RDKP_Build_Transaction(szTransactionKey, iCurTransactionID, szPlayer, iDKP, iGR, szComment);
end

function RDKP_Transaction_Raid(szPlayer, iDKP, iGR, szComment)
	-- This will be the single atomic operation that will be presented in the raid log.
	-- and communicated to the rest of the raid.
	-- szPlayer = who gets the amount
	-- iDKP = DKP amount increase
	-- iGR = GR amount increase
	-- These adjustments should be unique to themselves.
	if (not iDKP) then 
		iDKP = 0;
	end
	if (not iGR) then
		iGR = 0;
	end
	if ((iDKP == 0) and (iGR == 0)) then 
		return nil;
	end
	if (szComment == nil) then 
		szComment = "";
	end
	RDKP_Table_RaidInfo.RDKP_TransactionID = RDKP_Table_RaidInfo.RDKP_TransactionID + 1;
	return RDKP_Build_Transaction(RDKP_Table_RaidInfo.RDKP_RaidID, RDKP_Table_RaidInfo.RDKP_TransactionID, szPlayer, iDKP, iGR, szComment);
end


function RDKP_Build_Transaction(szRaidID, iTransID, szPlayer, iDKP, iGR, szComment)
	-- This will be the single atomic operation that will be presented in the raid log.
	-- and communicated to the rest of the guild.
	-- szRaidID = RaidID of the old raid.
	-- iTransID = Transaction ID for that raid.
	-- szPlayer = who gets the amount
	-- iDKP = DKP amount increase
	-- iGR = GR amount increase
	if (szRaidID == nil) then
		if szComment then 
			error("RDKP_Build_Transaction() szRaidID == nil & " .. szComment, 1);
		else
			error("RDKP_Build_Transaction() szRaidID == nil", 1);
		end
	end
	if (iTransID == nil) then
		if szComment then 
			error("RDKP_Build_Transaction() iTransID == nil & " .. szComment, 1);
		else
			error("RDKP_Build_Transaction() iTransID == nil", 1);
		end
	end
	if (szPlayer == nil) then
		if szComment then 
			error("RDKP_Build_Transaction() szPlayer == nil & " .. szComment, 1);
		else
			error("RDKP_Build_Transaction() szPlayer == nil", 1);
		end
	end
	if (iDKP == nil) then
		if szComment then 
			error("RDKP_Build_Transaction() iDKP == nil & " .. szComment, 1);
		else
			error("RDKP_Build_Transaction() iDKP == nil", 1);
		end
	end
	if (iGR == nil) then
		if szComment then 
			error("RDKP_Build_Transaction() iGR == nil & " .. szComment, 1);
		else
			error("RDKP_Build_Transaction() iGR == nil", 1);
		end
	end
	if (szComment == nil) then
		szComment = "";
		RDKP_Debug("RDKP_Build_Transaction() szComment == nil");
	end
	if (szRaidID == nil) then
		RDKP_StartRaid();
		if (RDKP_Table_RaidInfo["RDKP_RaidID"] == nil) then
			RDKP_Table_RaidInfo["RDKP_RaidID"] = RDKP_Build_RaidKeyID();
		end
		szRaidID = RDKP_Table_RaidInfo["RDKP_RaidID"] 
	end
	local szTransaction = RDKP_SZ_TRANSACTION_PREFIX;
	szTransaction = szTransaction .. szRaidID .. RDKP_SZ_FIELD_DELIMITER;
	szTransaction = szTransaction .. iTransID .. RDKP_SZ_FIELD_DELIMITER;
	szTransaction = szTransaction .. szPlayer .. RDKP_SZ_FIELD_DELIMITER;
	szTransaction = szTransaction .. iDKP .. RDKP_SZ_FIELD_DELIMITER;
	szTransaction = szTransaction .. iGR .. RDKP_SZ_FIELD_DELIMITER;
	szTransaction = szTransaction .. szComment .. RDKP_SZ_FIELD_DELIMITER;
	RDKP_Debug("RDKP_Build_Transaction: " .. szTransaction);
	return szTransaction;
end

function RDKP_Process_Transaction_Raid(szCaptured)
	RDKP_Debug("RDKP_Process_Transaction_Raid(" .. szCaptured .. ")");
	-- For transactions that are occuring during a raid, we were not pulling out the Transaction Prefix from our matching.
	-- Now we do this to ensure that we're just saving the raidID
	szPattern = "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER;
	-- Might want to do some verification of raidIDs at this point, but for now... 
	-- let's just make it work.
	local iStart, iEnd, szRaidID, iTransactionID, szPlayer, iDKP, iGR, szComment = string.find(szCaptured,  szPattern );
	if (not (iStart)) then
		error("ERROR: Could not parse: " .. szCaptured, 1);
	elseif (szPlayer == nil) then
		error("ERROR: Could parse: " .. szCaptured .. " But szPlayer == nil.", 1);
	end
	if ( not IsRaidLeader() ) then 
		RDKP_Table_RaidInfo.RDKP_RaidID = szRaidID;
		RDKP_Table_RaidInfo["RDKP_TransactionID"] = iTransactionID
	end
	return RDKP_Process_Transaction(szRaidID, iTransactionID, szPlayer, tonumber(iDKP), tonumber(iGR), szComment);
end

function RDKP_Process_Transaction_Historic(szCaptured)
	-- This is now a legacy function.  The new historic data is coming through the table pipe.
	-- The string passed in already has the transaction prefix pulled out, so we can just manage starting w/ the Raid ID.
	RDKP_Debug("RDKP_Process_Transaction_Historic(" .. szCaptured .. ")");
	szPattern = "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER;
	local iStart, iEnd, szRaidID, iTransactionID, szPlayer, iDKP, iGR, szComment = string.find(szCaptured,  szPattern );
	if (not (szRaidID)) then
		error("ERROR: Could not parse: " .. szCaptured, 1);
	elseif (szPlayer == nil) then
		error("ERROR: Could parse: " .. szCaptured .. " But szPlayer == nil.", 1);
	end
	return RDKP_Process_Transaction(szRaidID, iTransactionID, szPlayer, tonumber(iDKP), tonumber(iGR), szComment);
end

function RDKP_Process_Transaction(szRaidID, iTransactionID, szPlayer, iDKP, iGR, szComment)
	if (szPlayer == nil) then
		RDKP_Debug("RDKP_Process_Transaction() where szPlayer == nil... returning");
		return nil;
	end
	if (not iDKP) then
		iDKP = 0;
	end
	if (not iGR) then
		iGR = 0;
	end
	RDKP_Debug("RDKP_Process_Transaction(".. szRaidID .. ", " .. iTransactionID .. ", " .. szPlayer .. ", " .. iDKP .. ", " .. iGR .. ", " .. szComment .. ")");
	-- Fix if we're gettings a double TRANS prefix
	local iStart, iEnd, newRaidID = string.find(szRaidID, (RDKP_SZ_TRANSACTION_PREFIX .. "(.+)"));
	if iStart then
		szRaidID = newRaidID;
	end
	-- Record the transaction.
	if (not RDKP_Table_Raid_Register[szRaidID]) then
		RDKP_Table_Raid_Register[szRaidID] = {};
	end
	-- Need to check to see if it's a new transaction or if we're replacing an old one.
	-- If we're replacing an old one, then chances are we don't need to update the summary w/ that data.
	-- however, it's possible that the values are different then it's possible we need to reparse EVERYTHING.
	local b_newTransaction = false;
	if (RDKP_Table_Raid_Register[szRaidID][iTransactionID] == nil) then
		b_newTransaction = true;
	end
	RDKP_Table_Raid_Register[szRaidID][iTransactionID] = {
			["player"] = szPlayer, 
			["DKP"] = iDKP,
			["GR"] = iGR,
			["COMMENT"] = szComment,
		};
	-- At the time of the Transaction Processing, we should update the summaries.
	-- if the data is new.
	if b_newTransaction then
		RDKP_UpdateSummarywithTransaction(szPlayer, iDKP, iGR)
	end
	return true;
end

function RDKP_UpdateSummarywithTransaction(szPlayer, iDKP, iGR)
	-- Key point here though, is making sure that the master list is already as up-to-date as possible.
	-- Though argueably, if the summary isn't up to date, it will be once I get all the raid-data available onto the local list.
	-- The Player updated here also should be the Main Player, not necessarily the immediate player participating in the raid.
	if (not szPlayer) then
		RDKP_Debug("RDKP_Process_Transaction() where szPlayer == nil... returning");
		return nil;
	end
	if (not iDKP) then
		iDKP = 0;
	end
	if (not iGR) then
		iGR = 0;
	end
	RDKP_Debug("RDKP_UpdateSummarywithTransaction(".. szPlayer .. ", " .. iDKP .. ", " .. iGR ..")");
	if (not RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer]) then 
		-- Populate a character map w/ the existing character
		-- then continue
		RDKP_PlayerMap_AddNotMe(szPlayer);
	end
	local szMainCharacter = RDKP_PlayerMap_GetMain(szPlayer);
	if (not RDKP_Table_MasterPlayers[szMainCharacter]) then
		-- Incase the Character is not on the master list, update the master list and try.
		RDKP_PlayerMap_AddNotMe(szPlayer);
		RDKP_Table_MasterPlayers[szPlayer] = {
			["NAME"] = szPlayer, 
			["DKP"] = RDKP_Settings.InitialPlayerData.DKP, 
			["GR"] = RDKP_Settings.InitialPlayerData.GR,
			["PRI"] = nil,
		};
		szMainCharacter = szPlayer;
		RDKP_UpdateMasterPlayerList();
	end
	RDKP_Table_MasterPlayers[szMainCharacter]["DKP"] = RDKP_Table_MasterPlayers[szMainCharacter]["DKP"] + iDKP;
	RDKP_Table_MasterPlayers[szMainCharacter]["GR"] = RDKP_Table_MasterPlayers[szMainCharacter]["GR"] + iGR;
	RDKP_Table_MasterPlayers[szMainCharacter]["PRI"] = RDKP_AwardPlugins[RDKP_Settings.PrioritySystem].GetPlayerPriorityValue(szMainCharacter);
end
