-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------
-----------------------------------------------------------
-- Database Functions
-- 
--
function RDKP_ResetData()
	RDKP_Print(RDKP_SZ_DATA_RESET);
	-- Wipe out existing data
	RDKP_Table_CharacterMap = nil;
	RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME] =  nil;
	RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME] = {[RDKP_SZ_MYCHARACTERS] = nil,}
	RDKP_Table_MasterPlayers = nil;
	RDKP_Table_MasterPlayers = {}
	RDKP_Table_PlayersSorted = nil;
	RDKP_Table_PlayersSorted = {}
	RDKP_ResetSettingsLocal();
	-- Rebuild from scratch:
	RDKP_PlayerMap_Update();
	RDKP_UpdateMasterPlayerList();
	RDKP_BroadcastRaids();
	RDKP_ReparseTransactions();
	RDKP_SortMasterListbySetting();
	RDKP_SendSettingsUpdateRequest();
end

function RDKP_ResetRegister()
	RDKP_Table_Raid_Register = nil;
	RDKP_Table_Raid_Register = {}
end

function RDKP_UpdateRaidList()
	RDKP_Debug("RDKP_UpdateRaidList()");
	RDKP_Table_RaidPlayers = {};
	local iNumMembers = GetNumRaidMembers();
	if (iNumMembers > 0) then -- This sould be true/valid only if we're in a raid.  Otherwise it should be a party.
		RDKP_Debug("Raid Numbers: " .. iNumMembers );
		for iRaidMember = 1, iNumMembers, 1 do 
			local szPlayerName, rank, subgroup, level, class, fileName, szZone, online, isDead = GetRaidRosterInfo(iRaidMember);
			-- Populate the RaidList table w/ the character's MAINs.
			tinsert(RDKP_Table_RaidPlayers, RDKP_PlayerMap_GetMain(szPlayerName));
		end
	elseif (RDKP_Settings.AllowDKPFor5ManRuns) then 
		-- Parties are built weird compared to raids.  
		-- The is the index of everyone except self.
		iNumMembers = GetNumPartyMembers();
		if iNumMembers then 
			RDKP_Debug("Party Numbers: " .. iNumMembers );
			local unitid = "player";
			if not ( RDKP_SearchList(RDKP_Table_RaidPlayers, RDKP_PlayerMap_GetMain(UnitName(unitid)))) then
				tinsert(RDKP_Table_RaidPlayers, RDKP_PlayerMap_GetMain(UnitName(unitid)));
			end
			for i = 1, iNumMembers, 1 do
				unitid = "party" .. i;
				if not ( RDKP_SearchList(RDKP_Table_RaidPlayers, RDKP_PlayerMap_GetMain(UnitName(unitid)))) then
					tinsert(RDKP_Table_RaidPlayers, RDKP_PlayerMap_GetMain(UnitName(unitid)));
				end;
			end
		end
	end
end

function RDKP_Build_RaidKeyID()
	-- We need a way to have a consistent unique ID for each raid.
	-- This function should be used by the RL at the time the raid start is triggered.
	-- What will be unique to a given raid?
	-- RaidLeader & Date/Time at time of creation.
	-- We need to call this only when the raid is created.
	RDKP_Debug("RDKP_Build_RaidKeyID()");
	if (RDKP_Table_RaidInfo["RDKP_RaidID"] == nil) then
		local szName = UnitName("player");
		local szTempKey = szName .. "_" .. date(); -- NOTE: This is pulling the local system date, not the server date.
		RDKP_Table_RaidInfo["RDKP_RaidID"] = szTempKey;
		return szTempKey;
	else
		return RDKP_Table_RaidInfo["RDKP_RaidID"];
	end
end

function RDKP_ReparseTransactions()
	-- There may be times when the current MasterPlayerList isn't up-to-date w/ all the data available in local transactions.
	RDKP_Debug("RDKP_ReparseTransactions()");
	RDKP_RepairRaidIDs();
	-- First set all players DKP & GR values to 0
	if (not RDKP_Settings.InitialPlayerData) then
		RDKP_Settings.InitialPlayerData = RDKP_SystemDefaults.RDKP_Settings.InitialPlayerData;
	end
	for players, playerData in pairs(RDKP_Table_MasterPlayers) do
		playerData.DKP = RDKP_Settings.InitialPlayerData.DKP;
		playerData.GR = RDKP_Settings.InitialPlayerData.GR;
		playerData.PRI = nil;
	end
	-- Iterate through the RaidRegister and reparse that information.
	for raidID, v in pairs(RDKP_Table_Raid_Register) do
		RDKP_Debug("RID:" .. raidID);
		if (not RDKP_Table_Raid_Register[raidID].DESTROY) then
			for TransactionID, TransactionValue in pairs(v) do
				RDKP_UpdateSummarywithTransaction(TransactionValue["player"], TransactionValue["DKP"], TransactionValue["GR"]);
			end
		end
	end
end

function RDKP_GetPurgeDateFromString()
	RDKP_Debug("RDKP_GetPurgeDateFromString()");
	local szDate = RDKP_Settings.PurgeBeforeDate;
	if (szDate == nil) then 
		return nil;
	else
		-- expecting date : YYYY.MM.DD
		local tDate = {};
		local iStart, iEnd, iYear, iMonth, iDay =  string.find(szDate, RDKP_SZ_PATTERNDATE);
		if not (iYear == nil and iMonth == nil and iDay == nil) then 
			assert(iYear);
			assert(iMonth);
			assert(iDay);
			tDate.year = iYear;
			tDate.month = iMonth;
			tDate.day = iDay;
		else
			error("invalid Purge date in settings: " .. tostring(RDKP_Settings.PurgeBeforeDate));
			return nil;
		end
		RDKP_Debug(RDKP_TblToStr(tDate)) 
		tDate = time(tDate);
		RDKP_Debug(RDKP_TblToStr(tDate)) 
		return tDate;
	end
end

function RDKP_GetRaidDateFromRID(szRaidID)
	RDKP_Debug("RDKP_GetRaidDateFromRID(" .. tostring(szRaidID) .. ")");
	-- Retrieving the date from the RaidID
	-- Toon name followed by a possible '_' followed by what we expect to be some sort of date string.
	-- The real question is that if we feed that date string into a date() function, will we get back something we can use?
	local iStart, iEnd, iMonth, iDay, iYear = string.find(szRaidID, "^%a+_?(%d%d)/(%d%d)/(%d%d)");
	local tDate = {};
	if not (iYear == nil and iMonth == nil and iDay == nil) then 
		assert(iYear);
		assert(iMonth);
		assert(iDay);
		tDate.year = tonumber(iYear) + 2000;
		tDate.month = tonumber(iMonth);
		tDate.day = tonumber(iDay);
	else
		error("Unexpected date format in RaidID: " .. tostring(szRaidID));
		return nil;
	end
	local iDate = time(tDate);
	assert(iDate, "iDate == " .. tostring(iDate) .. "\n tDate == " .. RDKP_TblToStr(tDate));
	return iDate;
end

function RDKP_PurgeTransactions()
	RDKP_Debug("RDKP_PurgeTransactions()");
	local tPurgeDate = RDKP_GetPurgeDateFromString();
	if tPurgeDate == nil then
		return;
	end
	local iRaidDate;
	for szRaidID, tTransaction in pairs(RDKP_Table_Raid_Register) do
		iRaidDate = RDKP_GetRaidDateFromRID(szRaidID);
		if not (iRaidDate == nil) then
			if (iRaidDate < tPurgeDate) then
				RDKP_Print("Purging Raid:" .. szRaidID);
				RDKP_Debug("Purging Raid:" .. szRaidID);
				RDKP_Table_Raid_Register[szRaidID] = nil;
			else
				RDKP_Debug("iRaidDate: " .. tostring(iRaidDate) .. " => tPurgeDate: " .. tostring(tPurgeDate));
			end
		else
			RDKP_Debug(tostring(iRaidDate) .. " is NIL ");
		end
	end
end

function RDKP_BroadcastRaidTransactions(szRaidID)
	RDKP_Debug("RDKP_BroadcastRaidTransactions()" .. szRaidID);
	if RDKP_Table_Raid_Register[szRaidID] then
		if (RDKP_Table_Raid_Register[szRaidID].DESTROY == nil) then
			local topTable = {};
			topTable[szRaidID] = {};
			topTable[szRaidID] = RDKP_Table_Raid_Register[szRaidID];
			RDKP_Table_Broadcast("RDKP_Table_Raid_Register", topTable, "GUILD");
			topTable = nil;
		end
	end
end

function RDKP_RepairRaidIDs()
	RDKP_Debug("RDKP_RepairRaidIDs()");
	for raidID, raidTransactions in pairs(RDKP_Table_Raid_Register) do
		local iStart, iEnd, altraidID = string.find(raidID, RDKP_SZ_TRANSACTION_PREFIX .. "(.+)");
		if iStart then
			RDKP_Table_Raid_Register[altraidID] = raidTransactions;
			RDKP_Table_Raid_Register[raidID] = nil;
		end
	end
	RDKP_Debug("END - RDKP_RepairRaidIDs");
end

function RDKP_UpdateMasterPlayerList()
	local iLenGuildList = GetNumGuildMembers(true);
	local lstGuildNames = {}

	-- Validate external variables:
	if not (RDKP_SZ_REALMNAME == GetRealmName()) then
		RDKP_SZ_REALMNAME = GetRealmName();
	end
	-- Update the character map w/ the guild member status info.  
	if (not RDKP_Table_CharacterMap) then
		RDKP_Table_CharacterMap = {};
	end
	if (not RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME]) then
		RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME] = {};
	end
	
	local szGuildName, szRankName, iRankIndex = GetGuildInfo("player");
	for i = 0, iLenGuildList, 1 do
		local szName, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		tinsert(lstGuildNames, szName);
		if (szName) then 
			-- Character not on map:
			if not (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName]) then
				if string.lower(szName) == string.lower(UnitName("player")) then
					RDKP_PlayerMap_AddMe();
				else
					RDKP_PlayerMap_AddNotMe(szName);
				end
			end
			-- Now we should be sure the character info is there
			-- Update w/ the additional information to the table.
			if not (RDKP_Table_CharacterMap) then
				error("No CharacterMap");
			end
			if not (RDKP_SZ_REALMNAME) then
				error("No RealmName");
			end
			if not (szName) then
				error("No CharacterName");
			end
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName]["RANK"] = rank;
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName]["RANKINDEX"] = rankIndex;
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName]["LEVEL"] = level;
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName]["CLASS"] = class;
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName]["NOTE"] = note;
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName]["OFFICERNOTE"] = officernote;
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName]["STATUS"] = status;
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName]["GUILD"] = szGuildName;
			
			if (not RDKP_Table_MasterPlayers) then
				-- no Master table - need to create it.
				RDKP_Table_MasterPlayers = {};
			end
			if (not (RDKP_Table_MasterPlayers[szName])) then
				-- Build the info on the table
				if (not RDKP_Settings.InitialPlayerData) then
					RDKP_Settings.InitialPlayerData = RDKP_SystemDefaults.RDKP_Settings.InitialPlayerData;
				end
				RDKP_Table_MasterPlayers[szName] = {
					["NAME"] = szName, 
					["DKP"] = RDKP_Settings.InitialPlayerData.DKP, 
					["GR"] = RDKP_Settings.InitialPlayerData.GR,
					["PRI"] = 0,
				}
			end
			RDKP_Table_MasterPlayers[szName]["PRI"] = RDKP_AwardPlugins[RDKP_Settings.PrioritySystem].GetPlayerPriorityValue(szName);
		end
	end
	for szCharacterNameOnMap, v in pairs(RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME]) do
		if not (RDKP_SearchList(lstGuildNames, szCharacterNameOnMap)) then
			if type(RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szCharacterNameOnMap]) == "table" then
				-- If the character is NOT on the current guild roster, but is listed to be so in my data, then remove the guild info because it's incorrect.
				if (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szCharacterNameOnMap]["GUILD"] == szGuildName) then
					RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szCharacterNameOnMap]["GUILD"] =  "";
				end
			end 
		end
	end
	RDKP_SortMasterListbySetting();
end

function RDKP_SortMasterListbySetting()
	RDKP_Debug("RDKP_SortMasterListbySetting()");
	if (RDKP_Table_MasterPlayers) then
		RDKP_Table_PlayersSorted = RDKP_SortTableByField(RDKP_Table_MasterPlayers, RDKP_FilterBy.SortBy);
	end
end

function RDKP_SortTableByField(tbl, field)
	local new_tbl = {};
	for k, v in pairs(tbl) do
		tinsert(new_tbl, k);
	end
	table.sort(new_tbl,
		function(a1, a2)
			if (type(a1) == "string" and type(a2) == "number") then
				return (a1 < tostring(a2));
			elseif (type(a1) == "number" and type(a2) == "string") then
				return (tostring(a1) > a2);
			else 
				return (a1 < a2);
			end
		end
	);
	if field then 
		RDKP_Debug("Sorted by [" .. field .. "] List:");
		table.sort(new_tbl,
			function(a1, a2)
				if (type (tbl[a1][field]) == "string") then
					return (tbl[a1][field] < tbl[a2][field]);
				else
					if (tbl[a1][field] ==  tbl[a2][field]) then
						return (a1 < a2);
					else
						if (tbl[a1][field] == nil) then
							tbl[a1][field] = 0;
						end
						if (tbl[a2][field] == nil) then
							tbl[a2][field] = 0;
						end
						return (tbl[a1][field] > tbl[a2][field]);
					end
				end
			end
		)
	end
	return new_tbl;
end

function RDKP_Debug_OutputMasterList()
	for i, v in pairs(RDKP_Table_MasterPlayers) do
		RDKP_Debug(i);
		local szCharStats = v.NAME .. ": " .. v["DKP"] .. " / " .. v["GR"] .. " = " .. v["PRI"];
		RDKP_Debug(szCharStats);
	end
end

-----------------------------------------------------------
-- Player Data Functions
-- 
--
function RDKP_PlayerMap_Update()
	-- Update the list of the current characters for the player.
	RDKP_PlayerMap_AddMe();
	-- Now that we have a list, we want each of the characters on the list
	-- to point to the main character.
	-- This allows me to look at the character map, and use any character to assign points
	-- to the main already provided.
	for k, v in pairs(RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][RDKP_SZ_MYCHARACTERS]) do
		if v then
			if not RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][v] then
				RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][v] = {};
			end
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][v]["RDKP_SZ_MAINCHAR"] = RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][RDKP_SZ_MYMAINCHAR];
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][v]["RDKP_SZ_LASTUPDATE"]	= date(); -- Not using pretty date here because we need it for comparison purposes.  Need to check to see if I can get this to be a straight number.
		else
			RDKP_Debug("No Char found at key: " .. k)
		end
	end
	RDKP_PlayerMap_Broadcast();
end

function RDKP_PlayerMap_Broadcast()
	-- Publish all the available characters on the PlayerMap.
	RDKP_Debug("RDKP_PlayerMap_Broadcast()");
	-- TODO: Low Priority: Update this to us the new way and iterate through the char map on receipt.
	--RDKP_Table_Broadcast("RDKP_Table_CharacterMap", RDKP_Table_CharacterMap, "GUILD");
	RDKP_PlayerMap_Broadcast_OLD();
end

function RDKP_PlayerMap_Broadcast_OLD()
	-- Publish all the available characters on the PlayerMap.
	RDKP_Debug("RDKP_PlayerMap_Broadcast_OLD()");
	for k, v in pairs(RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME]) do
		if (k ~= RDKP_SZ_MYCHARACTERS) and (k ~= RDKP_SZ_MYMAINCHAR) and (v["RDKP_SZ_MAINCHAR"]) then
			if (not v["RDKP_SZ_LASTUPDATE"]) then
				v["RDKP_SZ_LASTUPDATE"] = date();
			end
			local szInfo = RDKP_SZ_CHAR_TRANSMIT_PREFIX .. k .. RDKP_SZ_FIELD_DELIMITER .. v["RDKP_SZ_MAINCHAR"] .. RDKP_SZ_FIELD_DELIMITER .. v["RDKP_SZ_LASTUPDATE"];
			if (v["HIDDEN"] == 1) then
				szInfo = szInfo .. RDKP_SZ_FIELD_DELIMITER .. RDKP_SZ_HIDE;
			else 
				szInfo = szInfo .. RDKP_SZ_FIELD_DELIMITER .. RDKP_SZ_SHOW;
			end
			RDKP_Guild_SendInfo(szInfo);
		end
	end
end

function RDKP_RecievePlayerMapEntry(szInfo)
	local iStart, iEnd, szEntry, szMain, iUpdate, szHide;
	iStart, iEnd, szEntry, szMain, iUpdate, szHide = string.find(szInfo, "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" ) 
	if szEntry then 
		if (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szEntry] == nil) then
	--		RDKP_Debug("RDKP_Table_CharacterMap." .. szEntry .. "== nil");
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szEntry] = {
				RDKP_SZ_MAINCHAR = szMain,
				RDKP_SZ_LASTUPDATE = iUpdate, -- Not using pretty date here because we need it for comparison purposes.
			};
		elseif (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szEntry]["RDKP_SZ_LASTUPDATE"] < iUpdate) then
	--		RDKP_Debug("szEntry != nil") 
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szEntry] = {
				RDKP_SZ_MAINCHAR = szMain,
				RDKP_SZ_LASTUPDATE  = iUpdate; -- Not using pretty date here because we need it for comparison purposes.
			}
		end
		if (szHide == RDKP_SZ_HIDE) then
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szEntry]["HIDDEN"] = 1;
		else
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szEntry]["HIDDEN"] = nil;
		end
	end
end

function RDKP_PlayerMap_AddMe()
	-- Setup a mapping for all characters associated w/ this account.
	-- First, get the name of the current character
	local szPlayer = UnitName("player");
	-- Check to see if there's a "MyCharacters" entry - 
	if (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME]) then 
		if (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][RDKP_SZ_MYCHARACTERS] == nil) then
			-- If not, create it, w/ the initial value of the current character.
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][RDKP_SZ_MYCHARACTERS] = {szPlayer};
			-- Also set the current character as the main for this account.
			RDKP_PlayerMap_SetAsMain();
		else
			-- If so, add the current character to the list.
			-- Check to see if the character is already there:
			local bOnList = nil;
			for k, v in pairs(RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][RDKP_SZ_MYCHARACTERS]) do
				if ( v == szPlayer) then 
					bOnList = true;
				end
			end
			if (not bOnList) then
				-- if not add him.
				tinsert(RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][RDKP_SZ_MYCHARACTERS], szPlayer);
			end
		end
	else 
		RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME] = {
			[RDKP_SZ_MYCHARACTERS] = {szPlayer}
		};
		RDKP_PlayerMap_SetAsMain();
	end
end

function RDKP_PlayerMap_AddNotMe(szName)
	-- Setup a mapping for all characters associated w/ this account.
	-- First, get the name of the current character
	RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szName] = {
		RDKP_SZ_LASTUPDATE = date(),
		RDKP_SZ_MAINCHAR = szName,
	}
end

function RDKP_PlayerMap_SetAsMain()
	-- Set this character as the main, to track all settings.
	-- May have to have a MyMain by guild.
	-- But for now, we're going to leave it like this.
	RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][RDKP_SZ_MYMAINCHAR] = UnitName("player")
	-- Update the guild w/ this information:
	RDKP_PlayerMap_Update();
end

function RDKP_PlayerMap_SetOtherAsMain(szPlayerMain, szPlayer)
	-- Set this character as the main, to track all settings.
	RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer]["RDKP_SZ_MAINCHAR"] = szPlayerMain;
	RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer]["RDKP_SZ_LASTUPDATE"] = date();
	-- Go through the list and update any others that had this current character as the main, and change it to the new character.
	for k,v in pairs(RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME]) do
		if not (k == szPlayer) then
			if (RDKP_PlayerMap_GetMain(k) == szPlayer) then
				RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][k]["RDKP_SZ_MAINCHAR"] = szPlayerMain;
			end
		end
	end
	-- Update the guild w/ this information:
	RDKP_PlayerMap_Update();
end

function RDKP_PlayerMap_GetMain(szPlayer)
	-- Returns a string of the MainCharacter's Name for any given character.
	-- Else, returns the Character name passed in.
	local szMain = szPlayer;
	if (RDKP_Table_CharacterMap) then
		if (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME]) then
			if (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer]) then
				if(RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer]["RDKP_SZ_MAINCHAR"]) then
					szMain = RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer]["RDKP_SZ_MAINCHAR"];
				end
			end
		end
	end
	return szMain;
end

function RDKP_GetPlayers()
	RDKP_Debug("RDKP_GetPlayers()");
	local szCharStats = "";
	for k, v in pairs(RDKP_Table_MasterPlayers) do
		szCharStats = szCharStats .. "\n" .. k .. ": " .. v["DKP"] .. " / " .. v["GR"] .. " = " .. v["PRI"];
	end
	return szCharStats;
end 

function RDKP_ShowCharacter(szPlayer)
	if (szPlayer) then
		if RDKP_FilterBy.ShowHidden then
			return 1;
		elseif RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer] then
			if RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer]["HIDDEN"]  then
				return nil;
			end
		end
	else
		return nil;
	end
	return 1;
end

function RDKP_Table_FilterOutAlts()
	-- Return a list/table of characters where the [szPlayer] == [szPlayer]["RDKP_SZ_MAINCHAR"]
	local new_table = {};
	local szPlayer;
	local szPlayerMain;
	if (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME]) then
		for szPlayer, v in pairs(RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME]) do
			szPlayerMain = RDKP_PlayerMap_GetMain(szPlayer);
			if (string.lower(szPlayer) == string.lower(szPlayerMain)) then
				tinsert(new_table, szPlayer);
			else
				RDKP_Debug("Filtering Out: " .. szPlayer);
			end
		end
	end
	return new_table;
end

function RDKP_Table_ReduceByList(tableBig, tableSmall)
	-- Take any 2 lists and reduce the values of the 1st arg by the value list in the 2nd
	-- Maintaining the sort of the 1st list.
--	RDKP_Debug("Big:" .. RDKP_TableGetSize(tableBig).. "\nSmall:" .. RDKP_TableGetSize(tableSmall) )
	local new_table = {};
	for k, v in pairs(tableBig) do
--		RDKP_Debug(k .. ": " .. v .. " & " .. RDKP_TableGetSize(tableSmall));
		if (RDKP_SearchList(tableSmall, v)) then
--			RDKP_Debug("Insert:" .. v) 
			tinsert(new_table, v);
		end
	end
	return new_table;
end

function RDKP_SearchList(tbl, sz)
--	RDKP_Debug("RDKP_SearchList()")
	if sz then 
--		RDKP_Debug("sz: " .. sz)
	end
	if ((RDKP_TableGetSize(tbl) > 0) and sz) then
--		RDKP_Debug("tbl (" .. RDKP_TableGetSize(tbl) ..  ") & sz")
		for k, v in pairs(tbl) do
--			RDKP_Debug("RDKP_SearchList(): " .. v)
			if (string.lower(v) == string.lower(sz)) then
				return true;
			end
		end
	end
	return nil;
end

function RDKP_Table_Broadcast(szTableName, tTable, szChannel)
	--	Args:
	--		id - text id passed to identify sending and recieving addon
	--		message - the text to be delivered
	--		method - "GLOBAL", "PARTY", "RAID", "GUILD", "BATTLEGROUND" or "WHISPER"
	--		target - player name for "WHISPER" target (Optional)
	--		priority - "ALERT", "NORMAL", "BULK" (Optional - Defaults to "NORMAL")
	local TableToSend = {};
	RDKP_Debug("TableToSend[".. szTableName .. "]");
	TableToSend[szTableName] = tTable;
	Telepathy.sendTable(RDKP_SZ_ADDON_PREFIX, TableToSend, "GUILD");
end

function RDKP_Table_BroadcastClose(tbl, szTableName)
	-- HASH<TABLENAME>!<DATE>!<HASHVALUE>!
	local szTableHash = RDKP_SZ_HASH .. szTableName .. RDKP_SZ_FIELD_DELIMITER
	if tbl.date then
		szTableHash = szTableHash .. tbl.date;
	end
	szTableHash = szTableHash .. RDKP_SZ_FIELD_DELIMITER .. RDKP_GetHashForTable(tbl) .. RDKP_SZ_FIELD_DELIMITER;
	RDKP_Guild_SendInfo(szTableHash);
end

function RDKP_Process_Table_Raid(szCapture, szSource)
	if (RDKP_Table_Temp[szSource] == nil) then
		RDKP_Table_Temp[szSource] = {};
	end
	local tTemp = RDKP_Table_Temp[szSource];
	RDKP_Process_Table(tTemp, szCapture);
end

function RDKP_Process_Table_Guild(szCapture, szSource)
	if (RDKP_Table_Temp[szSource] == nil) then
		RDKP_Table_Temp[szSource] = {};
	end
	RDKP_Process_Table(RDKP_Table_Temp[szSource], szCapture);
end

function RDKP_Process_Table(tParent, tCapture)
	-- tParent == the holding table
	-- tCapture == The contents that should be put into tParent
	tParent = tCapture;
	-- Now we have the table in the holding area, now we need to put in in the correct area of data storage.
	-- At this point we could be getting raid transaction data & settings from this area.
	-- Once we handle the data, we'll want to clear it out so that we don't hang on to bad data.
	if tParent.RDKP_Table_Raid_Register then
		RDKP_Debug("tParent.RDKP_Table_Raid_Register");
		-- iterative copy of all temp data into the main storage area:
		-- This should only add if there are missing pieces.
		RDKP_tcopy(RDKP_Table_Raid_Register, tParent.RDKP_Table_Raid_Register);
		-- Blanking out of temp area
		tParent.RDKP_Table_Raid_Register = nil;
		-- refresh Player info.
		RDKP_ReparseTransactions();
	elseif tParent.RDKP_Settings then
		RDKP_Debug("tParent.RDKP_Settings");
		if (tParent.RDKP_Settings.date) and ((RDKP_Settings.date == nil) or (tParent.RDKP_Settings.date > RDKP_Settings.date)) then
			-- Our settings are older 
			RDKP_tcopy(RDKP_Settings, tParent.RDKP_Settings);
			RDKP_PurgeTransactions();
		elseif (RDKP_Settings.date) and (tParent.RDKP_Settings.date < RDKP_Settings.date) then
			-- Our settings are newer.
			RDKP_SendSettingsUpdate();
		end
		tParent.RDKP_Settings = nil;
	else
		for k, v in pairs(tParent) do
			error("Unknown Table received: tParent[" .. k .. "]", 0);
		end
	end
end

function RDKP_Process_Hash_Guild(szCapture, szSource)
	if (RDKP_Table_Temp[szSource] == nil) then
		-- if the base value hasn't been created, then we're coming in at the wrong time.
		return; 
	end
	local tTemp = RDKP_Table_Temp[szSource];
	RDKP_Process_Hash(tTemp, szCapture);
end

function RDKP_Process_Hash(tbl, szCapture)
	-- TABLENAME!DATE!0000!
	local pattern = "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER;
	local iStart, iEnd, szTableName, iDate, inHash = string.find(szCapture, pattern);
	-- Are we getting a hash value in the szCapture?
	if inHash then 
		-- Make sure the date in our holding area and the date we're given match
		if ((tbl[szTableName].date and iDate and tbl[szTableName].date == iDate) or (not iDate and not tbl[szTableName].date)) then
			-- At this point, we're at the point we can spend the expense to get the hash.
			if inHash == RDKP_StringHash(RDKP_TblToStr(tbl[szTableName])) then
				-- We have the entire table being sent, and we're good to go on adopting that info.
				if szTableName == "RDKP_Settings" then
					RDKP_Debug("Table Name == RDKP_Settings")
--					RDKP_tcopy(RDKP_Settings, v[key]);
				else
					error("We have been sent a valid HASH for valid table: " .. szTableName .. " but no knowledge of how to handle it.", 1);
				end
			else
				error("table Hashes do not equate.\n" .. inHash .. " = " .. RDKP_StringHash(RDKP_TblToStr(tbl[szTableName])));
			end
		end
	end
end

function RDKP_AddZone(szZone)
	RDKP_Debug("RDKP_AddZone()");
	if (not RDKP_IsDKPOfficer()) then
		return nil;
	end
	if (szZone == nil) then
		szZone = GetRealZoneText();
	end
	-- First, check to ensure that the zone doesn't already exist on the DKP & GR lists:
	local found_DKP = 0;
	local found_GR = 0;
	for k, v in pairs(RDKP_Settings.DKP) do
		if k == szZone then
			found_DKP = 1;
		end
	end
	for k, v in pairs(RDKP_Settings.GR) do
		for y, m in pairs(v) do
			if y == szZone then
				found_GR = 1;
			end
		end
	end
	if found_DKP == 0 then 
		RDKP_Settings.DKP[szZone] = {};
	end
	if found_GR == 0 then
		RDKP_Settings_Temp.GR[1][szZone] = 0;
		RDKP_Settings_Temp.GR[2][szZone] = 0;
		RDKP_Settings_Temp.GR[3][szZone] = 0;
		RDKP_Settings_Temp.GR[4][szZone] = 0;
	end
--	return szZone;
end

function RDKP_AddBoss(szBoss, szZone)
	RDKP_Debug("RDKP_AddBoss()");
	if (not RDKP_IsDKPOfficer()) then
		return nil;
	end
	if szZone == nil then
		szZone = GetRealZoneText();
	end
	RDKP_AddZone(szZone);
	for k, v in pairs(RDKP_Settings.DKP[szZone]) do
		if (k == szBoss) then
--			return 1;
		end
	end
	RDKP_Settings_Temp.DKP[szZone][szBoss] = 0;
--	return 1;
end

function RDKP_tcopy(to, from)   -- "to" must be a table (possibly empty)
	for k,v in pairs(from) do
		if(type(v)=="table") then
			-- We don't want to wipe out sub-tables that may already contain information.
			if (to[k] == nil) or (not type(to[k]) == "table") then
				to[k] = {}
			end
			RDKP_tcopy(to[k], v);
		else
			to[k] = v;
		end
	end
end

function RDKP_EvalTables(tbl1, tbl2, bIgnoreDate)
	-- Take 2 tables and evaluate them for being equal by value.
	RDKP_Debug("RDKP_EvalTables");
	if bIgnoreDate then
		RDKP_tremovebyval(tbl1, "date");
		RDKP_tremovebyval(tbl2, "date");
	end
	if (RDKP_GetHashForTable(tbl1) == RDKP_GetHashForTable(tbl2)) then
		return 1;
	else
		return nil;
	end
end

function RDKP_StringHash(text)
	local counter = 1;
	local len = string.len(text); 
	for i = 1, len, 3 do 
		counter = math.fmod(counter*8161, 4294967279) +  -- 2^32 - 17: Prime!
			(string.byte(text,i)*16776193) +
			((string.byte(text,i+1) or (len-i+256))*8372226) +
			((string.byte(text,i+2) or (len-i+256))*3932164);
	end; 
	return math.fmod(counter, 4294967291); -- 2^32 - 5: Prime (and different from the prime in the loop)
end

function RDKP_GetHashForTable(tbl)
	if tbl.HASH then
		tbl.HASH = nil;
	end
	return RDKP_StringHash(RDKP_TblToStr(tbl, 0));
end

function RDKP_TblToStr(tbl, b_NewLine)
	if not (type(tbl) == "table") then
		return tostring(tbl);
	end
	local szNewLine = "";
	local szTable = "";
	if b_NewLine then
		szNewLine = "\n";
	end
	-- First get the keys into a reliable order.
	local tbl_KeySorted = RDKP_SortTableByField(tbl);
	-- Iterate through the sorted list of keys, and reference their values explicitly.
	for _, k in pairs(tbl_KeySorted) do
		szTable = szTable .. "[" .. k .. "]".. "=";
		if type(tbl[k]) == "table" then 
			szTable = szTable .. "{" .. RDKP_TblToStr(tbl[k], b_NewLine) .. "}" .. szNewLine;
		else
			szTable = szTable .. tostring(tbl[k]) .. szNewLine;
		end
	end
	return szTable;
end

-- tremovebyval: remove a table row given its value
function RDKP_tremovebyval(tab, val)
	RDKP_Debug("RDKP_tremovebyval(" .. tostring(tab) ..  ", " .. tostring(val) .. ")");
	if (not type(tab) == "table") then 
		error("tab: (" .. tostring(tab) .. ") not a valid table.", 0);
		return false;
	end
	if not (type(val) == "number" or type(val) == "string") then
		error("val: (" .. tostring(val) .. ") not a valid value to search for.", 0);
		return false;
	end
	for k, v in pairs(tab) do
		if (v == val) then
			table.remove(tab, k);
			return true;
		end
	end
	return false;
end

function RDKP_TableGetSize(tInput)
	local iTSize = 0;
	if type(tInput) == "table" then
		for k, v in pairs(tInput) do
			iTSize = iTSize + 1;
		end
	end
	return iTSize;
end
