-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------

-----------------------------------------------------------
-- Ratio Award Functions
-- 
-- First Register this table of functions for the plugin.
RDKP_AwardPlugins.RATIO = {}
-- TODO: Update Settings info here.
-- Next plugin settings that match the table name.
RDKP_Settings.RATIO = {
	["DKPMax"] = 300;
	["AdjustCoEfficient"] = .667;
}

function RDKP_AwardPlugins.RATIO.KhaosSettings()
	if (RDKP_Settings.RATIO == nil) then
		RDKP_Settings.RATIO = {
			["DKPMax"] = 300;
			["AdjustCoEfficient"] = .667;
		}
	end
	local tOptions = {};
	-- Setup the 1st Option found in above settings.
	local PluginOption1 = { 
		id = "DKPMax"; -- Unique within the Set
		text = "RATIO System: Max DKP Threshold";
		difficulty = 1;
		helptext = "This is the max DKP available for any given character.  Their DKP & GR will be adjusted by the Adjustment % on the next event that yields them DKP.  This should have little to no effect on their Priority.";
		callback = function(state) 
			if RDKP_IsDKPOfficer() then
				RDKP_Settings_Temp.RATIO.DKPMax = state.slider;
			end
		end;
		feedback = function(state) 
			if RDKP_IsDKPOfficer() then
				if ( state.checked ) then 
					return "";
				else
					return "";
				end
			else
				return "Sorry, you do not have authorization to change settings.";
			end
		end;
		type = K_SLIDER;
		setup = {
			-- If slider
			sliderMin = 100;
			sliderMax = 1000;
			sliderStep = 10;
			sliderSignificantDigits = 0;
			--sliderText = "Choose a value from " .. iMin .."-" .. iMax;
			sliderLow = "100";
			sliderHigh = "1000";
			--sliderDisplayFunc = function ( value ) return (math.floor(slidervalue*100)/100).."%";  end; 
		};
		default = {
			slider = RDKP_Settings.RATIO.DKPMax;  
		};
		disabled = {
			slider = RDKP_Settings.RATIO.DKPMax;
		}
	};
	-- Setup the 2st Option found in above settings.
	local PluginOption2 = { 
		id = "AdjustCoEfficient"; -- Unique within the Set
		text = "RATIO System: DKP Adjustment %";
		difficulty = 1;
		helptext = "This is the Amount DKP & GR will be adjusted to when their character crosses the Max DKP line..";
		callback = function(state) 
			if RDKP_IsDKPOfficer() then
				RDKP_Settings_Temp.RATIO.AdjustCoEfficient = (state.slider/100);
			end
		end;
		feedback = function(state) 
			if RDKP_IsDKPOfficer() then
				if ( state.checked ) then 
					return "";
				else
					return "";
				end
			else
				return "Sorry, you do not have authorization to change settings.";
			end
		end;
		type = K_SLIDER;
		setup = {
			-- If slider
			sliderMin = 10.0;
			sliderMax = 90.0;
			sliderStep = .1;
			sliderSignificantDigits = 1;
			--sliderText = "Choose a value from " .. 0 .."-" .. 1;
			--sliderLow = "100";
			--sliderHigh = "1000";
			sliderDisplayFunc = function ( value ) return (value) .. "%";  end; 
		};
		default = {
			slider = RDKP_Settings.RATIO.AdjustCoEfficient * 100;  
		};
		disabled = {
			slider = RDKP_Settings.RATIO.AdjustCoEfficient * 100;
		}
	};
	-- Submit the settings to Khaos.
	tinsert( tOptions, PluginOption1);
	tinsert( tOptions, PluginOption2 );
	local Khaos_RDKP_Plugin_Ratio = {
		id = "RDKP_Plugin_Ratio";
		text = "RDKP Ratio Plugin";
		helptext = "Settings specific to the RDKP Plugin: 'Ratio'";
		difficulty = 1;
		-- TODO: Figure out if we can get away w/ not enabling/disabling the addon.
--		callback = function(enabled) if ( enabled) then Sea.io.print("My Addon Activated!") end end;
		default = true;
		options = tOptions;
		-- TODO: LOW: May want to move all the slash command work here.
	}
	if Khaos.validateOptionSet(Khaos_RDKP_Plugin_Ratio) then
		Khaos.registerOptionSet("DKP", Khaos_RDKP_Plugin_Ratio);
	else
		RDKP_Debug("Khaos_RDKP_Plugin_Ratio INVALID");
	end


end

function RDKP_AwardPlugins.RATIO.GetRatio(szName)
	local priority = 0;
	if (szName) then
		if (RDKP_Table_MasterPlayers[szName]) then 
			priority = RDKP_Table_MasterPlayers[szName]["DKP"]/RDKP_Table_MasterPlayers[szName]["GR"];
			return priority;
		else
			error ("RDKP_Table_MasterPlayers[" .. szName .. "] not found.");
		end
	else
		error("RDKP_AwardPlugins.GetRatio(): nil", 1)
	end
end

function RDKP_AwardPlugins.RATIO.GetPlayerPriorityValue(szName)
	return RDKP_AwardPlugins.RATIO.GetRatio(szName);
end

function RDKP_AwardPlugins.RATIO.Loot(szItem, iQuality, szPlayerReceiving, szLocation)
	-- The function for when an item gets looted
	RDKP_Debug("RDKP_AwardPlugins.RATIO.Loot")
	if not szLocation then 
		szLocation = GetRealZoneText();
	end
	if (iQuality >= 1) then
		if not RDKP_Settings.GR then 
			RDKP_Print("RDKP_Settings.GR == NIL\n" .. RDKP_SZ_RESTORE_DEFAULTS);
			RDKP_Settings.GR = RDKP_SystemDefaults.RDKP_Settings.GR;
			RDKP_SendSettingsUpdateRequest();
		end
		if RDKP_Settings.GR then 
			if ((not RDKP_Settings.GR[iQuality]) and (RDKP_SystemDefaults.RDKP_Settings.GR[iQuality])) then
				RDKP_Print("RDKP_Settings.GR. " .. iQuality ..  " == NIL\n" .. RDKP_SZ_RESTORE_DEFAULTS);
				RDKP_Settings.GR[iQuality] = RDKP_SystemDefaults.RDKP_Settings.GR[iQuality];
				RDKP_SendSettingsUpdateRequest();
			end
			local iGR = 0;
			if RDKP_Settings.GR[iQuality][szLocation] then
				iGR = RDKP_Settings.GR[iQuality][szLocation];
				RDKP_Debug("Location: " .. szLocation .. " from quality table. iGR: " .. iGR);
			else
				RDKP_Debug("ERROR: Missing Location: " .. szLocation .. " from quality table.");
			end
			-- Need to check for shards at this point.  If there was a shard created, 
			-- then we need to refund the last transaction.
			if ( RDKP_SearchList(RDKP_Settings.DE_MATS, szItem) ) then
				-- Only return the last item of value.
				if RDKP_Last_Loot[szPlayerReceiving] then 
					iGR = -1 * RDKP_Last_Loot[szPlayerReceiving];
				else
					-- If we didn't have a last item, then set it for 0.  
					-- This is only a problem when RLs have changed hands between aquire & DE.
					iGR = 0;
				end
			elseif ( RDKP_SearchList(RDKP_Settings.LOOT_NOCHARGE, szItem) ) then
				-- For those items that won't have a cost associated with them... 
				-- Like ZG coins or scarabs, or... ?
				iGR = 0;
			end
			if (iGR == 0) or (iGR == nil) then
				RDKP_Debug("iGR 0" );
				return nil;
			elseif (tonumber(iGR) > 0) then
				RDKP_Last_Loot[szPlayerReceiving] = iGR;
			end
			local szTrans = RDKP_Transaction_Raid(szPlayerReceiving, 0, iGR, (szItem .. ": " .. iQuality .. "-" .. szLocation) );
			if RDKP_Table_RaidInfo.RDKP_bCurrentRaid then
				RDKP_Debug("Loot Send Info" );
				RDKP_Raid_SendInfo(szTrans);
			elseif RDKP_Settings.AllowDKPFor5ManRuns then
				RDKP_Party_SendInfo(szTrans);
			end
		else
			error("Catostrophic Failure on loot settings.  Defaults not able to be restored.", 1);
		end
	end 
end

function RDKP_AwardPlugins.RATIO.Kill(szMobName, szLocation)
	-- The function for when a mob dies.
	RDKP_Debug("RDKP_AwardPlugins.RATIO.Kill:");
	if szLocation then 
		RDKP_Debug(szLocation);
	end
	if not szMobName then
		return nil;
	end
	-- Move forward w/ allocating the DKP to each member.
	local iDKP = 0;
	if not RDKP_Settings.DKP then 
		RDKP_Print("RDKP_Settings.DKP == NIL\n" .. RDKP_SZ_RESTORE_DEFAULTS);
		RDKP_Settings.DKP = RDKP_SystemDefaults.RDKP_Settings.DKP;
		RDKP_SendSettingsUpdateRequest();
	end
	-- Check for DKP value of the given mob
	if RDKP_Settings.DKP then
		RDKP_Debug("RDKP_Settings.DKP");
		if ((not RDKP_Settings.DKP[szLocation]) and (RDKP_SystemDefaults.RDKP_Settings.DKP[szLocation])) then
			RDKP_Debug("RDKP_Settings.DKP." .. szLocation .. " == NIL\n" .. RDKP_SZ_RESTORE_DEFAULTS);
			RDKP_Settings.DKP[szLocation] = RDKP_SystemDefaults.RDKP_Settings.DKP[szLocation];
			RDKP_SendSettingsUpdateRequest();
		end
		if RDKP_Settings.DKP[szLocation] then
			if RDKP_Settings.DKP[szLocation][szMobName] then
				iDKP = RDKP_Settings.DKP[szLocation][szMobName];
			end
		end
	else
		error("Catostrophic Failure on DKP settings.  Defaults not able to be restored.", 1);
	end
	if (iDKP == 0) then
		RDKP_Debug("iDKP == 0");
		return;
	end
	local szRealZone = GetRealZoneText();
	RDKP_Debug(szRealZone);
	local b_Award = 1;
	RDKP_Debug("RDKP_AwardPlugins.RATIO.Kill -- START FOR");
	-- RAID:
	local iNumMembers = GetNumRaidMembers();
	if (iNumMembers) and (iNumMembers > 0) then 
		for i = 1, iNumMembers, 1 do
			-- Get the info for who's in the raid.
			local szName, rank, subgroup, level, class, fileName, szZone, online, isDead = GetRaidRosterInfo(i);
			-- Make sure the character was there at the time of the kill:
			RDKP_Debug("raid" .. i .. ": " .. szName .. " in " .. szZone .. " == " .. szRealZone );
			if ( RDKP_AwardPlugins.RATIO.CheckAwardSetting(szName, online, szZone) ) then
				if RDKP_AwardPlugins.RATIO.CheckDKPforThreshold(szName) then
					RDKP_AwardPlugins.RATIO.AdjustforThreshold(szName);
				end
				local szTemp = szRealZone;
				if szMobName then 
					szTemp = szTemp .. ": " .. szMobName;
				end
				local szTrans = RDKP_Transaction_Raid(szName, iDKP, 0, szTemp);
				if szTrans then 
					local bResults = RDKP_Raid_SendInfo(szTrans);
					if ( not bResults ) then
						-- Problem 
						error("DKP Not Awarded for " .. szName, 1);
					end
				end
			else
				RDKP_Print(szName .. RDKP_SZ_NO_KILL_AWARD);
			end
		end
	elseif (RDKP_Settings.AllowDKPFor5ManRuns) and (RDKP_Settings.AllowDKPFor5ManRuns > 0) then
	-- PARTY:
		iNumMembers = GetNumPartyMembers();
		-- Get info for self.
		local unitid = "player";
		local szName = UnitName(unitid);
		local szZone = GetRealZoneText();
		-- Execute DKP for self.
		if ( RDKP_AwardPlugins.RATIO.CheckAwardSetting(szName, 1, szZone) ) then
			if RDKP_AwardPlugins.RATIO.CheckDKPforThreshold(szName) then
				RDKP_AwardPlugins.RATIO.AdjustforThreshold(szName);
			end
			local szTemp = szRealZone;
			if szMobName then 
				szTemp = szTemp .. ": " .. szMobName;
			end
			local szTrans = RDKP_Transaction_Raid(szName, iDKP, 0, szTemp);
			if szTrans then 
				local bResults = RDKP_Party_SendInfo(szTrans);
				if ( not bResults ) then
					-- Problem 
					error("DKP Not Awarded for " .. szName, 1);
				end
			end
		else
			RDKP_Print(szName .. RDKP_SZ_NO_KILL_AWARD);
		end
		-- Then get info for Rest of the party.
		if iNumMembers then 
			for i = 1, iNumMembers, 1 do
				-- Get the info for who's in the party.
				-- Make sure the character was there at the time of the kill:
				unitid = "party" .. i;
				szName = UnitName(unitid);
				szZone = ""; -- TODO: HIGH: Need to get info on the party member's Zone 
				RDKP_Debug("party" .. i .. ": " .. szName .. " in " .. szZone .. " == " .. szRealZone );
				if ( RDKP_AwardPlugins.RATIO.CheckAwardSetting(szName, UnitIsConnected(unitid), szZone) ) then
					if RDKP_AwardPlugins.RATIO.CheckDKPforThreshold(szName) then
						RDKP_AwardPlugins.RATIO.AdjustforThreshold(szName);
					end
					local szTemp = szRealZone;
					if szMobName then 
						szTemp = szTemp .. ": " .. szMobName;
					end
					local szTrans = RDKP_Transaction_Raid(szName, iDKP, 0, szTemp);
					if szTrans then 
						local bResults = RDKP_Party_SendInfo(szTrans);
						if ( not bResults ) then
							-- Problem 
							error("DKP Not Awarded for " .. szName, 1);
						end
					end
				else
					RDKP_Print(szName .. RDKP_SZ_NO_KILL_AWARD);
				end
			end
		end
	end
	RDKP_Debug("RDKP_AwardPlugins.RATIO.Kill -- END FOR");
	return 1;
end

function RDKP_AwardPlugins.RATIO.Time()
	-- The function for each "tick" of awarding DKP for time in the raid.
	RDKP_Debug("RDKP_AwardPlugins.RATIO.Time()");
	local szRealZone = GetRealZoneText();
	if RDKP_GetAwardPriveledges() then 
		RDKP_Debug("Have Award Priveledges");
		-- RAID:
		local iNumMembers = GetNumRaidMembers();
		if (iNumMembers) and (iNumMembers > 0) then 
			RDKP_Debug("In Raid:");
			for i = 1, iNumMembers, 1 do
				local szPlayer, rank, subgroup, level, class, fileName, szZone, online, isDead = GetRaidRosterInfo(i);
				-- Make sure the character was there at the time of the kill:
				RDKP_Debug("raid" .. i .. ": " .. szName .. " in " .. szZone .. " == " .. szRealZone );
				if ( RDKP_AwardPlugins.RATIO.CheckAwardSetting(szName, online, szZone) ) then
					RDKP_Debug("Yes Award for Time.")
					if RDKP_AwardPlugins.RATIO.CheckDKPforThreshold(szName) then
						RDKP_AwardPlugins.RATIO.AdjustforThreshold(szName);
					end
					local szTrans = RDKP_Transaction_Raid(szName, RDKP_Table_RaidInfo.iDKPperTrans, 0, ("Time"));
					if szTrans then 
						RDKP_Debug("Transaction Ready.")
						local bResults = RDKP_Party_SendInfo(szTrans);
						if ( not bResults ) then
							-- Problem 
							error("DKP Not Awarded for " .. szName, 1);
						end
					end
				else
					RDKP_Print(szName .. RDKP_SZ_NO_TIME_AWARD);
				end
			end
		elseif (RDKP_Settings.AllowDKPFor5ManRuns) and (RDKP_Settings.AllowDKPFor5ManRuns > 0) then
		-- PARTY:
			RDKP_Debug("In Party:");
			iNumMembers = GetNumPartyMembers();
			-- Get info for self.
			local unitid = "player";
			local szName = UnitName(unitid);
			local szZone = GetRealZoneText();
			-- Execute DKP for self.
			if ( RDKP_AwardPlugins.RATIO.CheckAwardSetting(szName, 1, szZone) ) then
				RDKP_Debug("Yes Award for Time.")
				if RDKP_AwardPlugins.RATIO.CheckDKPforThreshold(szName) then
					RDKP_AwardPlugins.RATIO.AdjustforThreshold(szName);
				end
				local szTrans = RDKP_Transaction_Raid(szName, RDKP_Table_RaidInfo.iDKPperTrans, 0, "Time");
				if szTrans then 
					RDKP_Debug("Transaction Ready.")
					local bResults = RDKP_Party_SendInfo(szTrans);
					if ( not bResults ) then
						-- Problem 
						error("DKP Not Awarded for " .. szName, 1);
					end
				end
			else
				RDKP_Print(szName .. RDKP_SZ_NO_TIME_AWARD);
			end
			-- Then get info for Rest of the party.
			if iNumMembers then 
				for i = 1, iNumMembers, 1 do
					-- Get the info for who's in the party.
					-- Make sure the character was there at the time of the kill:
					unitid = "party" .. i;
					szName = UnitName(unitid);
					szZone = ""; -- TODO: HIGH: Need to get info on the party member's Zone 
					RDKP_Debug("party" .. i .. ": " .. szName .. " in " .. szZone .. " == " .. szRealZone );
					if ( RDKP_AwardPlugins.RATIO.CheckAwardSetting(szName, UnitIsConnected(unitid), szZone) ) then
						RDKP_Debug("Yes Award for Time.")
						if RDKP_AwardPlugins.RATIO.CheckDKPforThreshold(szName) then
							RDKP_AwardPlugins.RATIO.AdjustforThreshold(szName);
						end
						local szTrans = RDKP_Transaction_Raid(szName, RDKP_Table_RaidInfo.iDKPperTrans, 0, "Time");
						if szTrans then 
							RDKP_Debug("Transaction Ready.")
							local bResults = RDKP_Party_SendInfo(szTrans);
							if ( not bResults ) then
								-- Problem 
								error("DKP Not Awarded for " .. szName, 1);
							end
						end
					else
						RDKP_Print(szName .. RDKP_SZ_NO_TIME_AWARD);
					end
				end
			end
		else
			RDKP_Debug("No Raid, and Not supporting Party");
		end
	end
	return 1;
end

function RDKP_AwardPlugins.RATIO.CheckDKPforThreshold(szName)
	if szName then 
		RDKP_Debug("RDKP_AwardPlugins.RATIO.CheckDKPforThreshold("  .. szName .. ")");
	else
		RDKP_Debug("RDKP_AwardPlugins.RATIO.CheckDKPforThreshold(nil)");
		return nil;
	end
	if not RDKP_Table_MasterPlayers[szName] then 
		return nil;
	end
	if (RDKP_Table_MasterPlayers[szName]["GR"] > 1) then
		if (RDKP_Table_MasterPlayers[szName]["DKP"] > 1) then
			if (RDKP_Settings.RATIO.DKPMax > 1) and (RDKP_Table_MasterPlayers[szName]["DKP"] > RDKP_Settings.RATIO.DKPMax) then
				RDKP_Debug("DKPMax = " ..  RDKP_Settings.RATIO.DKPMax);
				RDKP_Debug(szName .. " DKP = " ..  RDKP_Table_MasterPlayers[szName]["DKP"]);
				return 1;
			end
		end
	end
	return nil
end

function RDKP_AwardPlugins.RATIO.AdjustforThreshold(szName)
	if szName then 
		RDKP_Debug("RDKP_AwardPlugins.RATIO.AdjustforThreshold("  .. szName .. ")");
	else
		RDKP_Debug("RDKP_AwardPlugins.RATIO.AdjustforThreshold(nil)");
		return nil;
	end
	local ThreshAdjustDKP = -1 * math.ceil((RDKP_Table_MasterPlayers[szName]["DKP"]) * (1 - RDKP_Settings.RATIO.AdjustCoEfficient));
	local ThreshAdjustGR = -1 * math.ceil((RDKP_Table_MasterPlayers[szName]["GR"]) * (1 - RDKP_Settings.RATIO.AdjustCoEfficient));
	local szTrans = RDKP_Transaction_Adjustment(szName, ThreshAdjustDKP, ThreshAdjustGR, "Threshold Adjustment");
	if szTrans then 
		local bResults
		if RDKP_Table_RaidInfo.RDKP_bCurrentRaid then 
			bResults = RDKP_Raid_SendInfo(szTrans);
		else
			bResults = RDKP_Party_SendInfo(szTrans);
		end
		if ( not bResults ) then
			-- Problem 
			error("Adjustment Not made for " .. szName, 1);
		end
	end
	if (ThreshAdjustDKP < 0) and (ThreshAdjustGR < 0) then
		if RDKP_AwardPlugins.RATIO.CheckDKPforThreshold(szName) then
			RDKP_AwardPlugins.RATIO.AdjustforThreshold(szName);
		end
	end
end

function RDKP_AwardPlugins.RATIO.CheckAwardSetting(szName, online, szZone)
	if (szZone == nil) or (szZone == "") then
		szZone = GetRealZoneText();
	end
	RDKP_Debug("RDKP_AwardPlugins.RATIO.CheckAwardSetting(" .. szName .. ", " .. szZone .. ")");
	local b_Award = 1;
	if ((b_Award == 1) and (not RDKP_Settings.EarnDKPWhenOffline)) then
		b_Award = online;
	end
	if ((b_Award == 1) and (not RDKP_Settings.EarnDKPWhenNotInZone)) then
		b_Award = (szZone == GetRealZoneText());
	end
	if ((b_Award == 1) and (not RDKP_Settings.EarnDKPWhenAFK)) then
		b_Award = not UnitIsAFK(szName);
	end
	return b_Award;
end
