-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------

-----------------------------------------------------------
-- Settings Functions
-- These functions will be used for managing settings.
--
function RDKP_IsDKPOfficer()
	-- Return true if current officer has the rights to grant somone additional DKP/GR.
	-- For right now, this is based purely off the ability to edit the officer's note.
	-- At some point, these should be able to be set in the options.
	if RDKP_b_Debug then
		return 1;
	else
		return CanEditOfficerNote();
	end
end

function RDKP_ResetSettingsLocal()
	RDKP_TableForAudit = RDKP_SystemDefaults.RDKP_TableForAudit;
	RDKP_Settings = RDKP_SystemDefaults.RDKP_Settings;
	RDKP_SendSettingsUpdateRequest();
end

function RDKP_SendSettingsUpdateRequest()
	local szSettingDate = RDKP_Settings.date;
	local szSettingsRequest = RDKP_SZ_SETTINGSREQUEST .. RDKP_SZ_FIELD_DELIMITER;
	if (szSettingDate) then
		szSettingsRequest = szSettingsRequest .. szSettingDate;
	end 
	szSettingsRequest = szSettingsRequest .. RDKP_SZ_FIELD_DELIMITER;
	RDKP_Guild_SendInfo(szSettingsRequest);
end

function RDKP_ReceiveSettingsUpdateRequest(szInfo)
	-- Recieve the request for new settings
	-- Matches the pattern:
	local szPattern = RDKP_SZ_FIELD_DELIMITER .. "(.-)" .. RDKP_SZ_FIELD_DELIMITER
	local iStart, iEnd, szSettingDate = string.find(szInfo, szPattern)
	if (not szSettingDate) then 
		if (RDKP_Settings.date) then
			RDKP_SendSettingsUpdate();
		end
	elseif ((szSettingDate) and (RDKP_Settings.date)) then
		if (szSettingDate < RDKP_Settings.date) then
			RDKP_SendSettingsUpdate();
		end
	end
end

function RDKP_SendSettingsUpdate()
	-- By the time this is called, we should already know if we have newer info
	-- than the requester.
	RDKP_Table_Broadcast("RDKP_Settings", RDKP_Settings, "GUILD");
--	RDKP_Table_BroadcastClose(RDKP_Settings, "RDKP_Settings");
end

function RDKP_SaveSettings()
	-- Saving the settings after they've been changed in the UI.
	-- Set the values from the holding area, to the final saved area.
	RDKP_Debug("RDKP_SaveSettings()");
	if RDKP_IsDKPOfficer() then
		RDKP_Debug("RDKP_IsDKPOfficer()");
		local bTablesEquate = RDKP_EvalTables(RDKP_Settings, RDKP_Settings_Temp, 1);
		if not bTablesEquate then
			RDKP_Print("Saving Settings");
			RDKP_Settings = {};
			RDKP_tcopy(RDKP_Settings, RDKP_Settings_Temp) ;
			RDKP_Settings.date = date();
			if Khaos then
				Khaos.refresh();
			end
			-- Finally, transmit settings changes to the guild.
			RDKP_SendSettingsUpdate();
			RDKP_PurgeTransactions();
		else
			RDKP_Print("No change in Settings.  Not Saved.");
		end
	end
end

function RDKP_ConvertSettings1xTo2x()
	if RDKP_GR_VALUE or RDKP_TABLE_DKP_VALUE then
		RDKP_Print("Converting Legacy(1.x) settings to 2.x")
		if RDKP_GR_VALUE then
			-- RDKP_GR_VALUE
			if RDKP_GR_VALUE["ff1eff00"] then
				RDKP_tcopy(RDKP_Settings["GR"][1], RDKP_GR_VALUE["ff1eff00"]);
			end
			if RDKP_GR_VALUE["ff0070dd"] then
				RDKP_tcopy(RDKP_Settings["GR"][2], RDKP_GR_VALUE["ff0070dd"]);
			end
			if RDKP_GR_VALUE["ffa335ee"] then
				RDKP_tcopy(RDKP_Settings["GR"][3], RDKP_GR_VALUE["ffa335ee"]);
			end
			if RDKP_GR_VALUE["ffff8000"] then
				RDKP_tcopy(RDKP_Settings["GR"][4], RDKP_GR_VALUE["ffff8000"]);
			end
			RDKP_GR_VALUE = nil;
		end
		-- RDKP_TABLE_DKP_VALUE
		if RDKP_TABLE_DKP_VALUE then
			RDKP_Settings["DKP"]["TIME"] = RDKP_TABLE_DKP_VALUE["hour"];
			for szZone, szBoss in pairs(RDKP_Settings["DKP"]) do
				if type(RDKP_Settings["DKP"][szZone]) == "table" then
					for k, v in pairs(RDKP_Settings["DKP"][szZone]) do
						if string.lower(k) == "default" then
							RDKP_Settings["DKP"][szZone][k] = RDKP_TABLE_DKP_VALUE["trash"];
						else
							RDKP_Settings["DKP"][szZone][k] = RDKP_TABLE_DKP_VALUE["kill"];
						end
					end
				end
			end
			RDKP_TABLE_DKP_VALUE = nil;
		end
	end
end

-----------------------------------------------------------
-- Khaos Functions
-- These functions will be used for managing settings using Khaos
-- 

-- TODO: Localize All Below
-- Setup a DKP folder in Khaos
function RDKP_Khaos_SetupFolder()
	-- Check if we've got Khaos available.
	if (not Khaos) then
		return nil;
	end
	RDKP_Debug("RDKP_Khaos_SetupFolder()");
	-- First Delete the old folder if it's there.
	Khaos.unregisterFolder("DKP");
	-- Now Create the folder setting again.
	-- This is to prevent it storing old data in Khaos.  I want this data only to come from my settings.
	local Khaos_DKPFolder = {
		id = "DKP";
		text = "DKP Addons";
		helptext = "These addons support DKP systems for your guild.";
		difficulty = 1;  -- 1 - Easy as Pie, 5 - Most people can handle it, 10 - Lua addicts only
	}
	-- Register the folder if it checks out.
	if (Khaos.validateFolder(Khaos_DKPFolder)) then
		Khaos.registerFolder(Khaos_DKPFolder);
	else
		RDKP_Debug("Khaos_DKPFolder INVALID");
		return nil;
	end
	return true;
end

-- Setup the RDKP OptionSet
function RDKP_Khaos_SetupOptionSet()
	if (not Khaos) then
		return nil;
	end
	RDKP_Debug("RDKP_Khaos_SetupOptionSet()");
	RDKP_Settings_Temp = {};
	RDKP_tcopy(RDKP_Settings_Temp, RDKP_Settings) ;
	-- TODO: Figure a way to display only/not allow editing if the present character doesn't have officer access.
	local Khaos_RDKPSet = {
		id = "RDKP_OptionSet";
		text = "Relational DKP";
		helptext = "DKP system that compares effort (DKP) with gear recieved (GR) to provide a relative comparision of guild members who raid.";
		difficulty = 1;
		-- TODO: Figure out if we can get away w/ not enabling/disabling the addon.
--		callback = function(enabled) if ( enabled) then Sea.io.print("My Addon Activated!") end end;
		default = true;
		options = RDKP_Khaos_SetupOptions();
		-- TODO: LOW: May want to move all the slash command work here.
--		commands = {
--			{KhaosSlashCommand},
--			{KhaosSlashCommand}, ...
--		}
	}
	-- Validate & register the OptionSet
	Khaos.unregisterOptionSet("RDKP_OptionSet");
	if Khaos.validateOptionSet(Khaos_RDKPSet) then
		Khaos.registerOptionSet("DKP" ,Khaos_RDKPSet);
	else
		RDKP_Debug("Khaos_RDKPSet INVALID");
	end
	return true;
end

--Setup the Options, return the table of all of the options.
function RDKP_Khaos_SetupOptions()
	if (not Khaos) then
		return nil;
	end
	RDKP_Debug("RDKP_Khaos_SetupOptions()");
	local tOptions = {};
	-- Add a header for the boolean options:
	tinsert( tOptions, {id = "RDKP_Header"; -- Unique within the Set
		text = "Relational DKP Options";
		helptext = "Relational DKP Options - to be set by officers only.";
		difficulty = 1;
		type = K_HEADER;
		});
	-- TODO: Add button for reset all to defaults and request update.
	-- Add the Boolean options that should just be check boxes:
	tinsert( tOptions, RDKP_Khaos_SaveSettingsButton());
	tinsert( tOptions, RDKP_AwardPluginKhaosSettingsSetup());
	tinsert( tOptions, RDKP_Khaos_SetupOption_Bool("AllowDKPFor5ManRuns", 
		"Allow DKP Recording For 5 Man Instance Runs", 
		"Check if you want to allow DKP to be recorded for going on 5 man runs.",
		false) );
	tinsert( tOptions, RDKP_Khaos_SetupOption_Bool("AllowDKPForOldWowInstances", 
		"Allow DKP Recording For old original Wow Instances", 
		"Check if you want to allow DKP to be recorded for going Original Wow content.",
		false) );
	tinsert( tOptions, RDKP_Khaos_SetupOption_Bool("OnlyOfficersCanRunRaids", 
		"Only Guild Officers can run Raids", 
		"Check if you only want Guild Officers to be able to record DKP for raids.",
		false) );
	tinsert( tOptions, RDKP_Khaos_SetupOption_Bool("EarnDKPWhenOffline", 
		"Allow DKP Earning when individuals are offline", 
		"Check if you want to allow DKP to be recorded for someone in the raid who happens to be offline.",
		true) );
	tinsert( tOptions, RDKP_Khaos_SetupOption_Bool("EarnDKPWhenNotInZone", 
		"Allow DKP Earning when individuals are not in the same zone", 
		"Check if you want to allow DKP to be recorded for someone in the raid who happens to be not in the same zone.\nMaybe they released after death so are at the graveyard.",
		false) );
	tinsert( tOptions, RDKP_Khaos_SetupOption_Bool("EarnDKPWhenAFK", 
		"Allow DKP Earning when individuals are away from their keyboard", 
		"Check if you want to allow DKP to be recorded for someone in the raid who happens to be Away from their keyboard.",
		false) );
	tinsert( tOptions, RDKP_Khaos_SetupOption_Bool("Reset Edit Fields", 
		"Reset the edit character value fields back to 0 upon saves.", 
		"Reset the edit character data fields back to 0 when you accept a change.",
		true) );
	tinsert( tOptions, {id = "RDKP_Inital_Player"; -- Unique within the Set
		text = "Initial Player Options.";
		helptext = "Adjusting the initial values of the system for new members.";
		difficulty = 1;
		type = K_HEADER;
		});
	local Initialoption1 = { id = "InitialDKP"; -- Unique within the Set
					text = "New Members Starting DKP:";
					difficulty = 1;
					helptext = "Adjust to change the amout of DKP a new member should start with.  This will adjust all players by this amount.";
					callback = function(state) 
						if RDKP_IsDKPOfficer() then
							RDKP_Settings_Temp.InitialPlayerData.DKP = state.slider;
						end
					end;
					feedback = function(state) 
						if RDKP_IsDKPOfficer() then
							return "All present and future guild members have their DKP adjusted by " .. state.slider .. ".";
						else
							return "Sorry, you do not have authorization to change settings.";
						end
					end;
					type = K_SLIDER;
					setup = {
						-- If slider
						sliderMin = 0;
						sliderMax = 100;
						sliderStep = 1;
						sliderSignificantDigits = 0;
						--sliderText = "Choose a value from " .. iMin .."-" .. iMax;
						sliderLow = 0;
						sliderHigh = 60;
						--sliderDisplayFunc = function ( value ) return (math.floor(slidervalue*100)/100).."%";  end; 
					};
					default = {
						slider = RDKP_Settings.InitialPlayerData.DKP;  
					};
					disabled = {
						slider = RDKP_Settings.InitialPlayerData.DKP;
					}
				};
	local Initialoption2 = { id = "InitialGR"; -- Unique within the Set
					text = "New Members Starting GR:";
					difficulty = 1;
					helptext = "Adjust to change the amout of GR a new member should start with.  This will adjust all players by this amount.";
					callback = function(state) 
						if RDKP_IsDKPOfficer() then
							RDKP_Settings_Temp.InitialPlayerData.GR = state.slider;
						end
					end;
					feedback = function(state) 
						if RDKP_IsDKPOfficer() then
							return "All present and future guild members have their GR adjusted by " .. state.slider .. ".";
						else
							return "Sorry, you do not have authorization to change settings.";
						end
					end;
					type = K_SLIDER;
					setup = {
						-- If slider
						sliderMin = 0;
						sliderMax = 100;
						sliderStep = 1;
						sliderSignificantDigits = 0;
						--sliderText = "Choose a value from " .. iMin .."-" .. iMax;
						sliderLow = 0;
						sliderHigh = 60;
						--sliderDisplayFunc = function ( value ) return (math.floor(slidervalue*100)/100).."%";  end; 
					};
					default = {
						slider = RDKP_Settings.InitialPlayerData.GR;  
					};
					disabled = {
						slider = RDKP_Settings.InitialPlayerData.GR;
					}
				};
	tinsert( tOptions, Initialoption1 );
	tinsert( tOptions, Initialoption2 );
	-- Add Header for DKP entries
	tinsert( tOptions, {id = "RDKP_Header_DKP"; -- Unique within the Set
		text = "DKP Values";
		difficulty = 1;
		helptext = "DKP Values: Amount earned when killing bosses and such.";
		type = K_HEADER;
		});
	-- Add sliders for each of the DKP entries
	local tOptionList = RDKP_Khaos_SetupOption_DKP();
	for k, v in pairs(tOptionList) do 
		tinsert( tOptions, v);
	end
	-- slider for DKP per hour.
	local DKPpHoption = { id = "TIME"; -- Unique within the Set
					text = "DKP per Hour";
					difficulty = 1;
					helptext = "Adjust to change the amout of DKP earned per hour.";
					callback = function(state) 
						if RDKP_IsDKPOfficer() then
							RDKP_Settings_Temp.DKP.TIME = state.slider;
						end
					end;
					feedback = function(state) 
						if RDKP_IsDKPOfficer() then
							return "Raiders earn " .. state.slider .. " DKP per hour.";
						else
							return "Sorry, you do not have authorization to change settings.";
						end
					end;
					type = K_SLIDER;
					setup = {
						-- If slider
						sliderMin = 0;
						sliderMax = 60;
						sliderStep = 1;
						sliderSignificantDigits = 0;
						--sliderText = "Choose a value from " .. iMin .."-" .. iMax;
						sliderLow = 0;
						sliderHigh = 60;
						--sliderDisplayFunc = function ( value ) return (math.floor(slidervalue*100)/100).."%";  end; 
					};
					default = {
						slider = RDKP_Settings.DKP.TIME;  
					};
					disabled = {
						slider = RDKP_Settings.DKP.TIME;
					}
				};
	tinsert(tOptions, DKPpHoption);
	-- button for adding new Zones
	tinsert( tOptions, RDKP_Khaos_AddNewZone());
	-- button for adding new bosses
	tinsert( tOptions, RDKP_Khaos_AddNewBoss());
	-- Add Header for GR entries
	tinsert( tOptions, {id = "RDKP_Header_GR"; -- Unique within the Set
		text = "GR Values";
		difficulty = 1;
		helptext = "GR Values: Amount spent when getting gear.";
		type = K_HEADER;
		});
	-- Add sliders for each of the GR entries.
	tOptionList = {};
	tOptionList = RDKP_Khaos_SetupOption_GR();
	for k, v in pairs(tOptionList) do 
		tinsert( tOptions, v);
	end
	-- Color code the names on the sliders by quality.

	return tOptions;
end

function RDKP_Khaos_SetupOption_Bool(szKey, szText, szHelpText, bDefault)
	local KhaosOption =  {
		id = szKey; -- Unique within the Set
		text = szText;
		helptext = szHelpText;
		value = 1;
		difficulty = 1;
		type = K_TEXT;
		-- TODO: setup proper callbacks for when this is checked.
		callback = function(state) 
			if RDKP_IsDKPOfficer() then
				if (state.checked) then
					RDKP_Settings_Temp[szKey] = 1;
				else
					RDKP_Settings_Temp[szKey] = 0;
				end
			end
		end;
		feedback = function(state) 
			if RDKP_IsDKPOfficer() then
				if ( state.checked ) then 
					return szText .. " is enabled.";
				else
					return szText .. " is disabled.";
				end
			else
				return "Sorry, you do not have authorization to change settings.";
			end
		end;
		check = true; 
		default = {
			checked = bDefault;
		};
		disabled = {
			checked = bDefault;
		};
	}
	return KhaosOption;
end

-- A function specifically handling the DKP Values
function RDKP_Khaos_SetupOption_DKP()
	local KhaosOptionList = {}
	for szZone, szBoss in pairs(RDKP_Settings.DKP) do
		-- Based on some new settings, search through the zones and only write the ones we care about
		-- If the user doesn't care about 5-mans or old WoW content, then don't write that info.
		if (RDKP_Khaos_TrackThisZone(szZone)) then
			if (type(szBoss) == "table") then
				for k, v in pairs(szBoss) do
					local KhaosOption =  {
						id = k; -- Unique within the Set
						text = szZone .. ":" .. k;
						difficulty = 1;
						helptext = "Adjust the DKP earned by killing " .. k .. " in " .. szZone;
						callback = function(state) 
							if RDKP_IsDKPOfficer() then
								RDKP_Settings_Temp["DKP"][szZone][k] = state.slider;
							end
						end;
						feedback = function(state) 
							if RDKP_IsDKPOfficer() then
								if ( state.checked ) then 
									return szZone .. ":" .. k .. " is enabled at " .. state.slider .. ".";
								else
									return szZone .. ":" .. k .. " is disabled.";
								end
							else
								return "Sorry, you do not have authorization to change settings.";
							end
						end;
						type = K_SLIDER;
						setup = {
							-- If slider
							sliderMin = 0;
							sliderMax = 100;
							sliderStep = 1;
							sliderSignificantDigits = 0;
							--sliderText = "Choose a value from " .. iMin .."-" .. iMax;
							sliderLow = 0;
							sliderHigh = 100;
							--sliderDisplayFunc = function ( value ) return (math.floor(slidervalue*100)/100).."%";  end; 
						};
						default = {
							slider = RDKP_Settings["DKP"][szZone][k];  
						};
						disabled = {
							slider = RDKP_Settings["DKP"][szZone][k];
						};
					}
					tinsert(KhaosOptionList, KhaosOption);
				end
			end
		end
	end
	return KhaosOptionList;
end

function RDKP_Khaos_SetupOption_GR()
	local KhaosOptionList = {}
	for iQuality, szZone in pairs(RDKP_Settings.GR) do
		if (type(szZone) == "table") then
			for k, v in pairs(szZone) do
				if (RDKP_Khaos_TrackThisZone(k)) then
					local KhaosOption =  {
						id = tostring(iQuality) .. ":" .. k; -- Unique within the Set
						text = "|c" .. RDKP_MAP_STRINGS.RARITY_INT[iQuality] .. k .. "|r";
						difficulty = 1;
						helptext = "Adjust the GR spent by getting |c" .. RDKP_MAP_STRINGS.RARITY_INT[iQuality] .. " quality|r Gear in " .. k;
						callback = function(state) 
							if RDKP_IsDKPOfficer() then
								RDKP_Settings_Temp["GR"][iQuality][k] = 0;
							end
						end;
						feedback = function(state) 
							if RDKP_IsDKPOfficer() then
								if ( state.checked ) then 
									return state.slider .. " GR spent by getting |c" .. RDKP_MAP_STRINGS.RARITY_INT[iQuality] .. " quality|r Gear in " .. k;
								else
									return "0 GR spent by getting |c" .. RDKP_MAP_STRINGS.RARITY_INT[iQuality] .. " quality|r Gear in " .. k;
								end
							else
								return "Sorry, you do not have authorization to change settings.";
							end
						end;
						type = K_SLIDER;
						setup = {
							-- If slider
							sliderMin = 0;
							sliderMax = 100;
							sliderStep = 1;
							sliderSignificantDigits = 0;
							--sliderText = "Choose a value from " .. iMin .."-" .. iMax;
							sliderLow = 0;
							sliderHigh = 100;
							--sliderDisplayFunc = function ( value ) return (math.floor(slidervalue*100)/100).."%";  end; 
						};
						default = {
							slider = RDKP_Settings["GR"][iQuality][k];  
						};
						disabled = {
							slider = RDKP_Settings["GR"][iQuality][k];
						};
					}
					tinsert(KhaosOptionList, KhaosOption);
				end
			end
		end
	end
	return KhaosOptionList;
end

function RDKP_Khaos_AddNewZone()
	local KhaosOption =  {
		id = "RDKP_AddZone"; -- Unique within the Set
		text = "Add Current Zone to RDKP.";
		difficulty = 1;
		helptext = "Adds the Zone that the current character is standing in to allow for DKP/GR recording in that instance/zone.  Values still need to be assigned.";
		callback = function() RDKP_AddZone(GetRealZoneText()); end;
		feedback = function() 
			return GetRealZoneText() .. " should now be on the zone list, if it wasn't there before.";
		end;
		type = K_BUTTON;
		setup = {
			-- If button
			buttonText = "Add Zone";
		};
		default = {
		};
		disabled = {
		};
		dependencies = {
		};
	}
	return KhaosOption;
end

function RDKP_Khaos_AddNewBoss()
	local KhaosOption =  {
		id = "RDKP_AddBoss"; -- Unique within the Set
		text = "Add Targeted Mob to RDKP.";
		difficulty = 1;
		helptext = "Adds the Mob that the current character is targeting to allow for DKP/GR recording of that boss.  Values still need to be assigned.";
		callback = function() RDKP_AddBoss(UnitName("target"), GetRealZoneText()); end;
		feedback = function() 
			return UnitName("target") .. " should now be on the boss list, if it wasn't there before.";
		end;
		type = K_BUTTON;
		setup = {
			-- If button
			buttonText = "Add Boss";
		};
		default = {
		};
		disabled = {
		};
		dependencies = {
		};
	}
	return KhaosOption;
end

function RDKP_Khaos_SaveSettingsButton()
	local KhaosOption =  {
		id = "RDKP_SaveSettings"; -- Unique within the Set
		text = "Save & Broadcast Settings Changes.";
		difficulty = 1;
		helptext = "If you have made changes, you need to broadcast those changes to the rest of the guild.";
		callback = function() 
				RDKP_Debug("RDKP_Khaos_SaveSettingsButton() Callback");
				RDKP_SaveSettings(); 
			end;
		feedback = function() 
			return "Settings Saved, and broadcast started.";
		end;
		type = K_BUTTON;
		setup = {
			-- If button
			buttonText = "Save Settings.";
		};
		default = {
		};
		disabled = {
		};
		dependencies = {
		};
	}
	return KhaosOption;
end

function RDKP_Khaos_TrackThisZone(szZone)
	if szZone == nil then
		RDKP_Debug("RDKP_Khaos_TrackThisZone(nil)");
		return nil;
	end
	RDKP_Debug("RDKP_Khaos_TrackThisZone(" .. szZone .. ")");
	if (RDKP_Settings.AllowDKPFor5ManRuns == 0) then
		-- No Party runs.
		if (RDKP_Settings.AllowDKPForOldWowInstances == 0) then
			-- No Old Wow Runs
			return RDKP_SearchList(RDKP_Table_Zones.Raid.TBC, szZone)
		else
			-- Raid only both New and Old content
			if (RDKP_SearchList(RDKP_Table_Zones.Raid.WOW, szZone)) then
				return true;
			elseif (RDKP_SearchList(RDKP_Table_Zones.Raid.TBC, szZone)) then
				return true;
			end
		end
	else
		-- Raids & 5 Mans
		if (RDKP_Settings.AllowDKPForOldWowInstances == 0) then
			-- No Old Wow Runs
			if (RDKP_SearchList(RDKP_Table_Zones.Raid.TBC, szZone)) then
				return true;
			elseif(RDKP_SearchList(RDKP_Table_Zones.Party.TBC, szZone)) then
				return true;
			end
		else
			-- both New and Old content
			if (RDKP_SearchList(RDKP_Table_Zones.Raid.WOW, szZone)) then
				return true;
			elseif (RDKP_SearchList(RDKP_Table_Zones.Raid.TBC, szZone)) then
				return true;
			elseif (RDKP_SearchList(RDKP_Table_Zones.Party.WOW, szZone)) then
				return true;
			elseif (RDKP_SearchList(RDKP_Table_Zones.Party.TBC, szZone)) then
				return true;
			end
		end
	end
	return nil;
end
