-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------
function RDKP_ShowUI()
	-- This is the Command to show the UI 
	RDKP_Debug("RDKP_ShowUI()");
	RelationalDKP_TopFrame_Overlay1_Text:SetText(RDKP_SZ_HEADER);
	if (RDKP_Table_RaidInfo["RecordRaid"] == 1) then
		RelationalDKP_TopFrame_LowerButtons_StartRaid:SetText(RDKP_SZ_END_RAID);
	else
		RelationalDKP_TopFrame_LowerButtons_StartRaid:SetText(RDKP_SZ_START_RAID);
	end
	if (IsPartyLeader() or IsRaidLeader()) then
		if RDKP_Settings.OnlyOfficersCanRunRaids then
			if not RDKP_IsDKPOfficer() then
				RelationalDKP_TopFrame_LowerButtons_StartRaid:Hide();
			else
				RelationalDKP_TopFrame_LowerButtons_StartRaid:Show();
			end
		else
			RelationalDKP_TopFrame_LowerButtons_StartRaid:Show();
		end
	else
		RelationalDKP_TopFrame_LowerButtons_StartRaid:Hide();
	end
	RelationalDKP_TopFrame:Show();
end

function RDKP_OnShow()
	RDKP_Debug("RDKP_OnShow()");
	RDKP_UpdateMasterPlayerList();
	RDKP_PriorityOnClick();
end

function RDKP_GetZones()
	local szZones = "";
	for i, v in pairs(RDKP_Table_Zones) do
		szZones = szZones .. "\n" .. v
	end
	return szZones;
end 

function RDKP_ShowSummary()
	RDKP_Debug("RDKP_ShowSummary()");
	-- Show the summary of players, along w/ their priority value.
	-- Sort Player list by Priority
	RDKP_SortMasterListbySetting();
	-- return that table - elements will need to be selectable.
	return RDKP_GetPlayers();
end

function RDKP_PopulatePriorityButtons()
	RDKP_Debug("RDKP_PopulatePriorityButtons()");
	RelationalDKP_DataFrame_Text:SetText("");
	local num = 0;
	-- Walk through of what we use here:
	-- RDKP_Table_PlayersSorted - is a global list only of all the characters we have on the Master table, however, sorted in the order we want.
	-- RDKP_RaidTellList - is the list of players in the raid that have sent tells.
	-- RDKP_Table_RaidPlayers - Is the list of plays in the raid.
	-- Check if we're in a raid:
	-- AND Check to see if we only want the current members of the Raid 
	-- If we're not in a raid it doesn't matter if the FilterBy is set, we want to show the full list.
	FauxScrollFrame_Update(RDKP_ScrollFrame,table.getn(RDKP_Table_PlayersSorted),15,RDKP_PRIORITY_FRAME_HEIGHT)
	local iPriorityListOffset = FauxScrollFrame_GetOffset(RDKP_ScrollFrame);
	if (RDKP_FilterBy.RaidMembers or RDKP_FilterBy.PartyMembers) then
		RDKP_UpdateRaidList();
		-- if so, then check to see if we only want those who will send tells (IE they want the current item up for bid)
		if (RDKP_FilterBy.PST) then
			-- Start accepting tells for bidding.  
			RDKP_Debug("RDKP_FilterBy.PST")
			RDKP_PriorityButtonsByFilter(RDKP_RaidTellList, iPriorityListOffset);
		else 
			RDKP_Debug("RDKP_FilterBy.Raid or PartyMembers")
			RDKP_PriorityButtonsByFilter(RDKP_Table_RaidPlayers, iPriorityListOffset);
		end	
	elseif (RDKP_Table_MasterPlayers) then 
		if (RDKP_FilterBy.HideAlts) then
			RDKP_PriorityButtonsByFilter(RDKP_Table_FilterOutAlts(), iPriorityListOffset);
		else
			RDKP_PriorityButtonsByFilter(nil, iPriorityListOffset);
		end
	else
		RDKP_Debug("RDKP_Table_MasterPlayers == nil");
	end
end

function RDKP_PriorityButtonsByFilter(tFilter, iStartingKey)
	-- tFilter is a table w/ the total list of characters we want to see in the UI. (IE having already filtered out those we don't want to see)
	RDKP_Debug("RDKP_PriorityButtonsByFilter()");
	-- Setup Header info.
	RDKP_PriorityButtonHeader_PlayerName:SetText("Player:")
	RDKP_PriorityButtonHeader_PlayerDKP:SetText("DKP: / GR: ")
	RDKP_PriorityButtonHeader_PlayerPriority:SetText("Priority: ")
	-- First, clear all the existing Priority buttons:
	for i = 1, RDKP_PRIORITY_BUTTON_QTY, 1 do
		local szName = "RDKP_PriorityButton" .. i .. "_PlayerName";
		local szDKP = "RDKP_PriorityButton" .. i .. "_PlayerDKP";
		local szPriority = "RDKP_PriorityButton" .. i .. "_PlayerPriority";
		local cbPlayerSelected = "RDKP_PriorityButton" .. i .. "_SelectCheckBox";
		getglobal(cbPlayerSelected):SetChecked(0);
		getglobal("RDKP_PriorityButton" .. i):Hide();
		getglobal(szName):SetText("");
		getglobal(szDKP):SetText("");
		getglobal(szPriority):SetText("");
	end
	RDKP_CharList = {};
	if (RDKP_TableGetSize(tFilter) > 0) then 
		RDKP_Debug("RDKP_TableGetSize(tFilter) == " .. RDKP_TableGetSize(tFilter) )
		RDKP_CharList = RDKP_Table_ReduceByList(RDKP_Table_PlayersSorted, tFilter);
	else
		RDKP_CharList = RDKP_Table_PlayersSorted;
	end
	local num = 0;
	RDKP_Debug("SizeOf(RDKP_CharList):" .. RDKP_TableGetSize(RDKP_CharList));
	for k, v in pairs(RDKP_CharList) do 
		if (k >= iStartingKey) then
			if (num < RDKP_PRIORITY_BUTTON_QTY) then 
				if (RDKP_ShowCharacter(v)) then
					num = num + 1;
					local szName = "RDKP_PriorityButton" .. num .. "_PlayerName";
					local szDKP = "RDKP_PriorityButton" .. num .. "_PlayerDKP";
					local szPriority = "RDKP_PriorityButton" .. num .. "_PlayerPriority";
					local cbPlayerSelected = "RDKP_PriorityButton" .. num .. "_SelectCheckBox";
					if RDKP_SearchList(RDKP_PlayerSelectList, v) then
						getglobal(cbPlayerSelected):SetChecked(1);
					end
					getglobal(szName):SetText(v);
		 			getglobal(szDKP):SetText(math.ceil(RDKP_Table_MasterPlayers[v]["DKP" ]) .. " / " .. math.ceil(RDKP_Table_MasterPlayers[v]["GR"]));
		 			getglobal(szPriority):SetText(RDKP_Table_MasterPlayers[v]["PRI" ]);
					getglobal("RDKP_PriorityButton" .. num):Show();
				end
	 		else
	 			return;
			end 
		end
	end
end 

function RDKP_ShowPriorityButtons(bShow)
	if (bShow) then 
		if RDKP_FilterBy.SortBy == "PRI" then
			RDKP_PriorityButtonHeader_PlayerName:SetFontObject("GameFontNormal");
			RDKP_PriorityButtonHeader_PlayerDKP:SetFontObject("GameFontNormal");
			RDKP_PriorityButtonHeader_PlayerPriority:SetFontObject("GameFontHighlight");
		elseif RDKP_FilterBy.SortBy == "NAME" then
			RDKP_PriorityButtonHeader_PlayerName:SetFontObject("GameFontHighlight");
			RDKP_PriorityButtonHeader_PlayerDKP:SetFontObject("GameFontNormal");
			RDKP_PriorityButtonHeader_PlayerPriority:SetFontObject("GameFontNormal");
		elseif RDKP_FilterBy.SortBy == "DKP" then
			RDKP_PriorityButtonHeader_PlayerName:SetFontObject("GameFontNormal");
			RDKP_PriorityButtonHeader_PlayerDKP:SetFontObject("GameFontHighlight");
			RDKP_PriorityButtonHeader_PlayerPriority:SetFontObject("GameFontNormal");
		elseif RDKP_FilterBy.SortBy == "GR" then
			RDKP_PriorityButtonHeader_PlayerName:SetFontObject("GameFontNormal");
			RDKP_PriorityButtonHeader_PlayerDKP:SetFontObject("GameFontHighlight");
			RDKP_PriorityButtonHeader_PlayerPriority:SetFontObject("GameFontNormal");
		end
		RDKP_PriorityUI:Show();
		Frame_FilterBy:Show();
	else
		RDKP_PriorityUI:Hide();
		Frame_FilterBy:Show();
		getglobal("RDKP_MemberDetailFrame"):Hide();
	end
end

function RDKP_PriorityOnClick()
	RDKP_Debug("RDKP_PriorityOnClick()");
	if (IsRaidLeader()) then
		RelationalDKP_TopFrame_LowerButtons_StartRaid:Show();
	else
		RelationalDKP_TopFrame_LowerButtons_StartRaid:Hide();
	end
	-- The reparse could cause serious performance issues as the raid transaction list gets bigger.
	-- So I've commented it out for now.  If we're getting data mis-matches then I'll add it back in.
	-- However, now w/ getting the raid transaction data in full tables, and reparsing the table at that point,
	-- I believe we'll cut down the need for reparsing on the refresh.
	RDKP_ReparseTransactions();
	RDKP_SortMasterListbySetting();
	RDKP_PopulatePriorityButtons();
	RDKP_ShowPriorityButtons(true);
end

function RDKP_RaidStartOnClick()
	RDKP_Debug("RDKP_RaidStartOnClick()");
	RDKP_ShowPriorityButtons(false);
	if (RDKP_Table_RaidInfo.RDKP_RaidID == nil) or not (RDKP_Table_RaidInfo["RecordRaid"]) then
		-- Starting raid.
		RDKP_StartRaid();
		RelationalDKP_TopFrame_LowerButtons_StartRaid:SetText(RDKP_SZ_END_RAID);
		RelationalDKP_DataFrame_Text:SetText(RDKP_SZ_RAID_START);
	elseif (RDKP_Table_RaidInfo.RDKP_RaidID) then
		-- Ending raid.
		RDKP_EndRaid();
		RelationalDKP_TopFrame_LowerButtons_StartRaid:SetText(RDKP_SZ_START_RAID);
		RelationalDKP_DataFrame_Text:SetText(RDKP_SZ_RAID_END);
	end
end

function RDKP_SetAsMainOnClick()
	RDKP_Debug("RDKP_SetAsMainOnClick()");
	if RDKP_IsDKPOfficer() then
		RDKP_GR_VALUE_TEMP = RDKP_GR_VALUE ;
		RDKP_TABLE_DKP_VALUE_TEMP = RDKP_TABLE_DKP_VALUE;
	end
	RDKP_ShowPriorityButtons(false);
	RelationalDKP_DataFrame_Text:SetText(RDKP_SZ_SET_MAIN);
	RDKP_PlayerMap_SetAsMain();
end

function RDKP_PriorityButton_OnClick(arg1)
	RDKP_Debug("RDKP_PriorityButton_OnClick(" .. tostring(arg1) .. ")");
	-- If you have DKP Officer rights
	if (RDKP_IsDKPOfficer()) then
		-- Get the size of the Player Checked list
		local itSelectSize = RDKP_TableGetSize(RDKP_PlayerSelectList);
		-- Update the audit table w/ the character clicked on.
		RDKP_TableForAudit.PlayerName = getglobal(this:GetName() .. "_PlayerName"):GetText();
		-- Left Click will be DKP/GR adjustment (single or group)
		if (arg1 == "LeftButton") then 
			-- display pop-up for adding DKP or GR to the character selected.
			RDKP_HideCharacterButton_OnShow();
			-- if there's nothing or only 1 check on the list, do it the old way:
			if (itSelectSize < 2) then
				-- If there's no name in the frame, then don't show the frame.:
				if (RDKP_TableForAudit.PlayerName) then
					RDKP_MemberDetailName:SetText(RDKP_TableForAudit.PlayerName);
					RDKP_MemberDetailNameMulti:SetText("");
					RDKP_MemberDetailDKPText:SetText(RDKP_Table_MasterPlayers[RDKP_TableForAudit.PlayerName]["DKP"]);
					RDKP_MemberDetailGRText:SetText(RDKP_Table_MasterPlayers[RDKP_TableForAudit.PlayerName]["GR"]);
					RDKP_MemberDetailPRIText:SetText(RDKP_Table_MasterPlayers[RDKP_TableForAudit.PlayerName]["PRI"]);
					RDKP_MemberDetailFrame:Show();
				end
			else
				local szPlayerSelectList = "";
				for k, v in pairs(RDKP_PlayerSelectList) do
					szPlayerSelectList = szPlayerSelectList .. "\n" .. v;
				end
				RDKP_TableForAudit.PlayerName = RDKP_PlayerSelectList;
				RDKP_MemberDetailNameMulti:SetText(szPlayerSelectList);
				RDKP_MemberDetailName:SetText("Many ->");
				RDKP_MemberDetailDKPText:SetText("Varies");
				RDKP_MemberDetailGRText:SetText("Varies");
				RDKP_MemberDetailPRIText:SetText("Varies");
				RDKP_MemberDetailFrame:Show();
			end
		else
			-- Right click will handle setting mains.
			if (arg1 == "RightButton") then 
				if (itSelectSize < 2) then
					-- If there aren't anyone extra on the list, then there won't be alot todo here.  
				else
					-- Set the various checked characters to the Rt-clicked as main.
					-- TODO: need to setup some popup for this before being able to actually execute.
					-- Pop-up to say "Make 'PlayerName' as Main for all Checked characters"
				end
			end
		end
	end
end

function RDKP_DKPSettingButton_OnClick(arg1)
	RDKP_Debug("RDKP_DKPSettingButton_OnClick()");
	-- If you have DKP Officer rights
	if (RDKP_IsDKPOfficer()) then
		-- display pop-up for adding DKP or GR to the character selected.
		RDKP_Setting_Selected = getglobal(this:GetName() .. "_PlayerName"):GetText();
		-- If there's no name in the frame, then don't show the frame.:
		if ((RDKP_Setting_Selected) and (RDKP_Setting_Selected ~= "DKP Type:")) then
			RDKP_Debug("RDKP_Setting_Selected = " .. RDKP_Setting_Selected);
			StaticPopup_Show("ADJUST_DKP_SETTING");
		end
	end
end

function RDKP_GRSettingButton_OnClick(arg1)
	RDKP_Debug("RDKP_DKPSettingButton_OnClick()");
	-- If you have DKP Officer rights
	if (RDKP_IsDKPOfficer()) then
		-- display pop-up for adding DKP or GR to the character selected.
		RDKP_SettingTableForAdjustment.SettingName = getglobal(this:GetName() .. "_ZoneName"):GetText();
		-- If there's no name in the frame, then don't show the frame.:
		if (RDKP_SettingTableForAdjustment.SettingName) then
			RDKP_SettingDetailName:SetText(RDKP_SettingTableForAdjustment.SettingName);  
			RDKP_SettingDetailGreenText:SetText(RDKP_GR_VALUE_TEMP["ff1eff00"][RDKP_SettingTableForAdjustment.SettingName]);
			RDKP_SettingDetailBlueText:SetText(RDKP_GR_VALUE_TEMP["ff0070dd"][RDKP_SettingTableForAdjustment.SettingName]);
			RDKP_SettingDetailEpicText:SetText(RDKP_GR_VALUE_TEMP["ffa335ee"][RDKP_SettingTableForAdjustment.SettingName]);
			RDKP_SettingTableForAdjustment["ff1eff00"] = RDKP_GR_VALUE_TEMP["ff1eff00"][RDKP_SettingTableForAdjustment.SettingName];
			RDKP_SettingTableForAdjustment["ff0070dd"] = RDKP_GR_VALUE_TEMP["ff0070dd"][RDKP_SettingTableForAdjustment.SettingName];
			RDKP_SettingTableForAdjustment["ffa335ee"] = RDKP_GR_VALUE_TEMP["ffa335ee"][RDKP_SettingTableForAdjustment.SettingName];
			RDKP_SettingDetailFrame:Show();
		end
	end
end

function RDKP_ToggleSorting()
	if RDKP_FilterBy.SortBy == "PRI" then
		RDKP_FilterBy.SortBy = "NAME";
		RDKP_PriorityOnClick();
		return;
	elseif RDKP_FilterBy.SortBy == "NAME" then
		RDKP_FilterBy.SortBy = "DKP"; 
		RDKP_PriorityOnClick();
		return;
	elseif RDKP_FilterBy.SortBy == "DKP" then
		RDKP_FilterBy.SortBy = "GR";
		RDKP_PriorityOnClick();
		return;
	elseif RDKP_FilterBy.SortBy == "GR" then
		RDKP_FilterBy.SortBy = "PRI";
		RDKP_PriorityOnClick();
		return;
	end
end

function RDKP_ToggleHideCharacter()
	local szPlayer = RDKP_MemberDetailName:GetText();
	if RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer] then
		if (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer].HIDDEN) then
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer].HIDDEN = nil;
			RDKP_MemberHideCharacterButton:SetText(RDKP_SZ_HIDE_CHAR);
		else
			RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer].HIDDEN = 1;
			RDKP_MemberHideCharacterButton:SetText(RDKP_SZ_SHOW_CHAR);
		end
		RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer].RDKP_SZ_LASTUPDATE = date();
	else
		RDKP_PlayerMap_AddNotMe(szPlayer);
		RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer].HIDDEN = 1;
		RDKP_MemberHideCharacterButton:SetText(RDKP_SZ_SHOW_CHAR);
		RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer].RDKP_SZ_LASTUPDATE = date();
	end
end

function RDKP_HideCharacterButton_OnShow()
	local szPlayer = RDKP_MemberDetailName:GetText();
	if RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer] then
		if (RDKP_Table_CharacterMap[RDKP_SZ_REALMNAME][szPlayer].HIDDEN) then
			RDKP_MemberHideCharacterButton:SetText(RDKP_SZ_SHOW_CHAR);
		else
			RDKP_MemberHideCharacterButton:SetText(RDKP_SZ_HIDE_CHAR);
		end
	else
		RDKP_MemberHideCharacterButton:SetText("");
	end
end

function RDKP_ShowRaids_OnClick()
	RDKP_Debug("RDKP_ShowRaids_OnClick()");
	getglobal("RDKP_MemberDetailFrame"):Hide();
	-- Populate the values of the RaidKeys.
	if RDKP_IsDKPOfficer() then
		RDKP_DestroyRaid:Show();
		RDKP_PurgeRaids:Show();
	else
		RDKP_DestroyRaid:Hide();
		RDKP_PurgeRaids:Hide();
	end
	RDKP_PopulateRaids();
	RDKP_RaidRegisterFrame:Show();
end

function RDKP_PopulateRaids()
	RDKP_Debug("RDKP_PopulateRaids()");
	local iRRSize = RDKP_TableGetSize(RDKP_Table_Raid_Register);
	RDKP_FauxScrollFrame_Update(RDKP_RaidID_ScrollFrame, iRRSize, 15, 15);
	local iRaidListOffset = FauxScrollFrame_GetOffset(RDKP_RaidID_ScrollFrame);
	RDKP_Debug("iRRSize: " .. iRRSize .. "\niRaidListOffset: " .. iRaidListOffset);
	for i = 1, 20, 1 do 
		getglobal("RDKP_RaidIDButton" .. i .. "_RaidKey"):SetText("");
	end
	local j = 1;
	local i = 1;
	local jMax = iRaidListOffset + 21;
	local tRaidKey = {};
	tRaidKey = RDKP_SortTableByField(RDKP_Table_Raid_Register);
	if (RDKP_Table_Raid_Register) then 
		for k, RaidID in pairs(tRaidKey) do 
			if (RDKP_Table_Raid_Register[RaidID].DESTROY) then
				if not (RDKP_FilterBy.ShowDestroyedRaids) then
					break;
				else
					if ((j >= iRaidListOffset) and (j < jMax)) then
						if (i < 21) then
							getglobal("RDKP_RaidIDButton" .. i .. "_RaidKey"):SetText(RaidID);
						else
							break;
						end
						i = i + 1;
					end
					j = j + 1;
				end
			else
				if ((j >= iRaidListOffset) and (j < jMax)) then
					if (i < 21) then
						getglobal("RDKP_RaidIDButton" .. i .. "_RaidKey"):SetText(RaidID);
					else
						break;
					end
					i = i + 1;
				end
				j = j + 1;
			end
		end
	end
end

function RDKP_RaidButton_OnClick(arg1)
	-- Someone clicked on the raid ID button.
	-- So we're going to want to list the Raid info on the other frame.
	RDKP_Debug("RDKP_RaidButton_OnClick()");
	local szRaidID = getglobal(this:GetName() .. "_RaidKey"):GetText();
	RDKP_Debug("szRaidID = " .. tostring(szRaidID));
	local szTemp = "";
	if (RDKP_Table_Raid_Register) then
		RDKP_Debug("RDKP_Table_Raid_Register");
		if (RDKP_Table_Raid_Register[szRaidID]) then
			RDKP_Debug(szRaidID);
			if (RDKP_Table_Raid_Register[szRaidID].DESTROY) then
				RDKP_Debug("DESTROYED");
				szTemp = szTemp .. "DESTROYED\n";
			else
				RDKP_Debug("NOT DESTROYED");
				table.sort(RDKP_Table_Raid_Register[szRaidID]);
				for k, TransFields in pairs(RDKP_Table_Raid_Register[szRaidID]) do
					if (TransFields.player) then
						szTemp = szTemp .. TransFields.player .. ": ";
					end
					if (TransFields.DKP) then
						szTemp = szTemp .. " DKP: " .. TransFields.DKP;
					end
					if (TransFields.GR) then
						szTemp = szTemp .. "  GR: " .. TransFields.GR .. "\n";
					else
						szTemp = szTemp .. "\n";
					end
					if (TransFields.COMMENT) then
						szTemp = szTemp .. "     " .. TransFields.COMMENT .. "\n";
					end
				end
			end
		end
	end
	RDKP_RaidRegisterFrame_Transactions_Body:SetText(szTemp);
	RDKP_Volitile_RAIDID = szRaidID;
end

RDKP_tParent = {};

function RDKP_Settings_OnShow()
	-- On the showing of the Settings Frame
	-- Populate the values of the left hand side.
	RDKP_Settings_Temp = {};
	RDKP_tcopy(RDKP_Settings_Temp, RDKP_Settings) ;
	RDKP_Populate_SettingFrame(RDKP_Settings_Temp, "RDKP_Settings");
	
end

function RDKP_SettingButton_OnClick(arg1)
	-- Populate the right side box w/ the values found under the Setting Key clicked on, on the left-hand side.
	RDKP_Debug("RDKP_SettingButton_OnClick()");
	-- Get the values that were set during OnShow or since the previous setting operations.
	local szTitle = getglobal("RDKP_SettingFrame_KeyFrame_Title"):GetText()
	-- Get the object referred to in the title button.
	local szSetting = getglobal(this:GetName() .. "_SettingKey"):GetText();
	local szChild = szTitle .. "[" .. szSetting .. "]";
	-- TODO: This section needs to be adjusted. HOW?!?
	local tChild = RDKP_tParent[table.maxn(RDKP_tParent)][szSetting]
	if (type(tChild) == "table") then 
		RDKP_Populate_SettingFrame(tChild, szChild);
	else
		RDKP_ValueButton_OnClick(this);
	end
end

function RDKP_ValueButton_OnClick(this)
	-- Bring up edit box to change the value.
	RDKP_Debug("RDKP_ValueButton_OnClick()");
	local iStart, iEnd, iNum = string.find(this:GetName(), "(%d)");
	local szSetting = getglobal("RDKP_SettingFrame_KeyFrame_KeyButton" .. iNum .. "_SettingKey"):GetText();
	local Setting_ToChange = RDKP_tParent[table.maxn(RDKP_tParent)][szSetting];
	-- tChild will be the setting that needs to be adjusted.
	-- Display an edit box over the button... 
	local bNumber = 1;
	if (string.lower(type(Setting_ToChange)) == "string") then
		bNumber = 0;
	end
	RDKP_Debug("type(Setting_ToChange): " .. string.lower(type(Setting_ToChange)) .. bNumber);	
	RDKP_AdjustSettingValue(szSetting, Setting_ToChange, bNumber);
end

function RDKP_Populate_SettingFrame(tParent, szTitle)
	-- First, clear all the buttons.
	for j=1, 20, 1 do 
		getglobal("RDKP_SettingFrame_KeyFrame_KeyButton" .. j .. "_SettingKey"):SetText("");
		getglobal("RDKP_SettingFrame_KeyFrame_ValueButton" .. j .. "_SettingKey"):SetText("");
	end
	-- Get the text of the button clicked on.
	RDKP_Debug("RDKP_Populate_SettingFrame(" .. szTitle .. ")");
	table.insert(RDKP_tParent, tParent);
	if table.maxn(RDKP_tParent) < 2 then
		-- if There aren't any additional Parents than the initial, then the top button should be disabled.
		getglobal("RDKP_SettingFrame_KeyFrame_Title"):Disable();
	else
		getglobal("RDKP_SettingFrame_KeyFrame_Title"):Enable();
	end
	local i = 0;
	RDKP_Debug("tParent = " .. type(tParent));
	if (type(tParent) ==  "table") then
		getglobal("RDKP_SettingFrame_KeyFrame_Title"):SetText(szTitle);
		for k,v in pairs(tParent) do
			i = i + 1;
			if i < 21 then 
				getglobal("RDKP_SettingFrame_KeyFrame_KeyButton" .. i .. "_SettingKey"):SetText(tostring(k));
				if (not (type(v) == "table")) then
					getglobal("RDKP_SettingFrame_KeyFrame_ValueButton" .. i .. "_SettingKey"):SetText(tostring(v));
				else
					getglobal("RDKP_SettingFrame_KeyFrame_ValueButton" .. i .. "_SettingKey"):SetText(tostring(k));
				end
			end
		end
	end
end

function RDKP_Refresh_SettingFrame()
	-- TODO: Low: Add offsets for scroll bar functionality.
	local tParent = table.remove(RDKP_tParent);
	local szTitle = getglobal("RDKP_SettingFrame_KeyFrame_Title"):GetText();
	RDKP_Populate_SettingFrame(tParent, szTitle);
end

function RDKP_KeyFrame_Title_OnClick(arg1)
	RDKP_Debug("RDKP_KeyFrame_Title_OnClick()");
	-- Get the values that were set during OnShow or since the previous setting operations.
	if table.maxn(RDKP_tParent) > 1 then
		local szTitle = getglobal("RDKP_SettingFrame_KeyFrame_Title"):GetText()
		-- Get the object referred to in the title button.
		local iStart, iEnd, szBase, szSub = string.find(szTitle, "(.+)%[(.-)%]");
		table.remove(RDKP_tParent);
		local tParent = table.remove(RDKP_tParent);
		if (not tParent) then
			error("RDKP_KeyFrame_Title_OnClick() no tParent Found.", 1);
		else
			RDKP_Populate_SettingFrame(tParent, szBase);
		end
	end
end

function RDKP_ResetButton_OnClick()
	RDKP_Debug("RDKP_ResetButton_OnClick()");
	RDKP_Settings_Temp = {};
	RDKP_tcopy(RDKP_Settings_Temp, RDKP_Settings);
end

function RDKP_SaveButton_OnClick()
	RDKP_Debug("RDKP_SaveButton_OnClick()");
	RDKP_SaveSettings();
end

function RDKP_AddKeyButton_OnClick()
	RDKP_Debug("RDKP_AddKeyButton_OnClick()");
	-- Add a key to the table referenced in the header.
	-- RDKP_AdjustSettingValue("Add new Key string", "", 0);
end

function RDKP_AddValueButton_OnClick()
	RDKP_Debug("RDKP_AddValueButton_OnClick()");
	-- Add a keyless value to the table referenced in the header
	-- RDKP_AdjustSettingValue("Add new unkeyed Value ", "", 0);
end

function RDKP_AdjustSettingValue(szTitle, szInitialValue, bExpectNumber)
	-- Set the title to what we want it to be.
	-- Set the initial value in the edit box
	-- TODO: Re-write to deal w/ the template.
	RDKP_Debug("RDKP_AdjustSettingValue()");
	if (bExpectNumber == 1) then 
		RDKP_Debug("bExpectNumber: " .. bExpectNumber);
		RDKP_ValueAdjust_bExpectNumber:SetText("1");
	else
		RDKP_ValueAdjust_bExpectNumber:SetText("");
	end
	if szTitle then 
		RDKP_Debug("szTitle: " .. szTitle);
		RDKP_ValueAdjust_Request:SetText(szTitle);
	end
	if szInitialValue then
		RDKP_Debug("szInitialValue: " .. szInitialValue);
		RDKP_ValueAdjust_EditBox:SetText(szInitialValue);
	end
	RDKP_ValueAdjust:Show();
end

function RDKP_RetrieveValue()
	local rValue = nil;
	RDKP_Debug("RDKP_RetrieveValue()");
	rValue = RDKP_ValueAdjust_EditBox:GetNumber();
	if rValue == nil then 
		rValue = RDKP_ValueAdjust_EditBox:GetText();
	end
	-- TODO: this next line should be on the receiving side of whoever is calling for the dialog box.
	RDKP_tParent[table.maxn(RDKP_tParent)][RDKP_ValueAdjust_Request:GetText()] = rValue;
	return rValue;
end

function RDKP_PriorityButton_Checkbox_OnClick(arg1)
	RDKP_Debug("RDKP_PriorityButton_Checkbox_OnClick()" .. tostring(arg1:GetName()));
	local iStart, iEnd, szParent = string.find(arg1:GetName(), "(.-)_SelectCheckBox")
	local szNameChecked = getglobal(szParent .. "_PlayerName"):GetText();
	RDKP_Debug("szNameChecked: " .. tostring(szNameChecked));
	if szNameChecked then
		if ( arg1:GetChecked() ) then
			RDKP_Debug(szNameChecked .. " Checked");
			tinsert(RDKP_PlayerSelectList, szNameChecked);
			PlaySound("igMainMenuOptionCheckBoxOn");
		else
			RDKP_Debug(szNameChecked .. " UnChecked");
			RDKP_tremovebyval(RDKP_PlayerSelectList, szNameChecked);
			PlaySound("igMainMenuOptionCheckBoxOff");
		end
	end
end

function RDKP_SelectAll_OnClick()
	RDKP_Debug("RDKP_SelectAll_OnClick()");
	if (RDKP_EvalTables(RDKP_PlayerSelectList, RDKP_CharList, true)) then
		for k, v in pairs(RDKP_CharList) do 
			RDKP_tremovebyval(RDKP_PlayerSelectList, v);
		end
	else
		RDKP_tcopy(RDKP_PlayerSelectList, RDKP_CharList);
	end
	RDKP_PopulatePriorityButtons();
end

StaticPopupDialogs["SAVE_AUDIT"] = {
	-- TODO: Localize here
	-- Dunno if we can.
	text = format(TEXT("Save Adjustment to DKP and/or GR values?"), "RDKP"),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		if type(RDKP_TableForAudit.PlayerName) == "string" then
			local szTrans = RDKP_Transaction_Adjustment(RDKP_TableForAudit.PlayerName, RDKP_TableForAudit.DKP, RDKP_TableForAudit.GR, "Adjustment");
			if szTrans then 
				local bResults = RDKP_Guild_SendInfo(szTrans);
				if ( not bResults ) then
					-- Problem 
					RDKP_Debug("DKP Not Awarded for " .. szName);
				end
			end
		else
			if type(RDKP_TableForAudit.PlayerName) == "table" then
				for k, v in pairs(RDKP_TableForAudit.PlayerName) do 
					local szTrans = RDKP_Transaction_Adjustment(v, RDKP_TableForAudit.DKP, RDKP_TableForAudit.GR, "Adjustment");
					if szTrans then 
						local bResults = RDKP_Guild_SendInfo(szTrans);
						if ( not bResults ) then
							-- Problem 
							RDKP_Debug("DKP Not Awarded for " .. szName);
						end
					end
				end
			end
		end
		-- TODO: Provide defect info 
		-- Fix for Defect: ()
		if (RDKP_Settings.ResetEditFields == 1) then
			RDKP_TableForAudit.DKP = 0;
			RDKP_TableForAudit.GR = 0;
		end
		-- Hide the frame.
		RDKP_TableForAudit = RDKP_SystemDefaults.RDKP_TableForAudit;
		RDKP_MemberDetailFrame:Hide();
		RDKP_PriorityOnClick();
		RDKP_AdjustDKPText:SetText("")
		RDKP_AdjustGRText:SetText("");
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["ADJUST_DKP"] = {
	text = TEXT("Adjust DKP: "),
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 10,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustDKPText:SetText(tostring(editBox:GetText()));
		RDKP_TableForAudit["DKP"] = tonumber(editBox:GetText());
	end,
	OnShow = function()
		if (RDKP_TableForAudit.DKP == nil) then
			RDKP_TableForAudit.DKP = 0;
		end
		getglobal(this:GetName().."WideEditBox"):SetText(RDKP_TableForAudit.DKP);
		getglobal(this:GetName().."WideEditBox"):SetFocus();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustDKPText:SetText(tostring(editBox:GetText()));
		RDKP_TableForAudit["DKP"] = tonumber(editBox:GetText());
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["ADJUST_GR"] = {
	text = TEXT("Adjust GR: "),
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 10,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustGRText:SetText(tostring(editBox:GetText()));
		RDKP_TableForAudit.GR = tonumber(editBox:GetText());
	end,
	OnShow = function()
		if (RDKP_TableForAudit.GR == nil) then
			RDKP_TableForAudit.GR = 0;
		end
		getglobal(this:GetName().."WideEditBox"):SetText(tostring(RDKP_TableForAudit.GR));
		getglobal(this:GetName().."WideEditBox"):SetFocus();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		RDKP_AdjustGRText:SetText(tostring(editBox:GetText()));
		RDKP_TableForAudit.GR = tonumber(editBox:GetText());
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["PURGE_RAIDS"] = {
	text = TEXT("Purge All Raids before this date: (YYYY.MM.DD)\n<WARNING> You cannot undo this action. Raids prior to this date will be removed from the database."),
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 10,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		local szDate = editBox:GetText();
		if (szDate == "nil" or szDate == "") then
			RDKP_Settings.PurgeBeforeDate = nil;
		else
			local iStart, iEnd, iYear, iMonth, iDay = string.find(tostring(szDate), RDKP_SZ_PATTERNDATE);
			assert(iYear);
			assert(iMonth);
			assert(iDay);
			RDKP_Settings.PurgeBeforeDate = tostring(szDate);
		end
		RDKP_Settings.date = date();
		RDKP_SendSettingsUpdate();
		RDKP_PurgeTransactions()
	end,
	OnShow = function()
		getglobal(this:GetName().."WideEditBox"):SetText(tostring(RDKP_Settings.PurgeBeforeDate));
		getglobal(this:GetName().."WideEditBox"):SetFocus();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		local szDate = editBox:GetText();
		if (szDate == "nil" or szDate == "") then
			RDKP_Settings.PurgeBeforeDate = nil;
		else
			local iStart, iEnd, iYear, iMonth, iDay = string.find(tostring(szDate), RDKP_SZ_PATTERNDATE);
			assert(iYear);
			assert(iMonth);
			assert(iDay);
			RDKP_Settings.PurgeBeforeDate = tostring(szDate);
		end
		RDKP_Settings.date = date();
		RDKP_SendSettingsUpdate();
		RDKP_PurgeTransactions()
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

StaticPopupDialogs["DESTROY_RAID"] = {
	-- Writes from the temp table holding 1 setting to the temp table of all settings.
	-- TODO: Localize here
	-- Dunno if we can.
	text = format(TEXT("Are You sure you want to Destroy the Raid?\nWARNING: This cannot be undone."), "RDKP"),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		RDKP_Table_Raid_Register[RDKP_Volitile_RAIDID] = {};
		RDKP_Table_Raid_Register[RDKP_Volitile_RAIDID].DESTROY = 1;
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

-- Function to handle the update of manually calculated scrollframes.  Used mostly for listings with an indeterminate number of items
function RDKP_FauxScrollFrame_Update(frame, numItems, numToDisplay, valueStep, button, smallWidth, bigWidth, highlightFrame, smallHighlightWidth, bigHighlightWidth )
	-- If more than one screen full of skills then show the scrollbar
	RDKP_Debug("RDKP_FauxScrollFrame_Update");
	local frameName = frame:GetName();
	RDKP_Debug(frameName);
	local scrollBar = getglobal( frameName.."ScrollBar" );
	local showScrollBar;
	if ( numItems > numToDisplay ) then
		frame:Show();
		showScrollBar = 1;
	else
		scrollBar:SetValue(0);
		frame:Hide();
	end
	if ( showScrollBar ) then
		RDKP_Debug("Frame Visible");

		local scrollChildFrame = getglobal( frameName.."ScrollChildFrame" );
		local scrollUpButton = getglobal( frameName.."ScrollBarScrollUpButton" );
		local scrollDownButton = getglobal( frameName.."ScrollBarScrollDownButton" );
		local scrollFrameHeight = 0;
		local scrollChildHeight = 0;

		if ( numItems > 0 ) then
			scrollFrameHeight = (numItems - numToDisplay) * valueStep;
			scrollChildHeight = numItems * valueStep;
			if ( scrollFrameHeight < 0 ) then
				scrollFrameHeight = 0;
			end
			scrollChildFrame:Show();
		else
			scrollChildFrame:Hide();
		end
		scrollBar:SetMinMaxValues(0, scrollFrameHeight); 
		scrollBar:SetValueStep(valueStep);
		scrollChildFrame:SetHeight(scrollChildHeight);
		
		-- Arrow button handling
		if ( scrollBar:GetValue() == 0 ) then
			scrollUpButton:Disable();
		else
			scrollUpButton:Enable();
		end
		if ((scrollBar:GetValue() - scrollFrameHeight) == 0) then
			scrollDownButton:Disable();
		else
			scrollDownButton:Enable();
		end
		
		-- Shrink because scrollbar is shown
		if ( highlightFrame ) then
			highlightFrame:SetWidth(smallHighlightWidth);
		end
		if ( button ) then
			for i=1, numToDisplay do
				getglobal(button..i):SetWidth(smallWidth);
			end
		end
	else
		RDKP_Debug("Frame Not Visible");

		-- Widen because scrollbar is hidden
		if ( highlightFrame ) then
			highlightFrame:SetWidth(bigHighlightWidth);
		end
		if ( button ) then
			for i=1, numToDisplay do
				getglobal(button..i):SetWidth(bigWidth);
			end
		end
	end
	return showScrollBar;
end
