-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------

-- When the AddOn loads...
function RDKP_OnLoad()
	-- Register the game events neccesary for our functionality
	-- Only registering the global events that we care about on a constant basis.
	this:RegisterEvent("PLAYER_LOGIN");
	this:RegisterEvent("GUILD_ROSTER_UPDATE");
	-- Register Slash Commands:
	SLASH_RDKP_SHOW1 = "/rdkp";
	SlashCmdList["RDKP_SHOW"] = RDKP_ShowUI;
	SLASH_RDKP_RESET1 = "/rdkp_resetall";
	SlashCmdList["RDKP_RESET"] = RDKP_ResetData;
	SLASH_RDKP_BOOMERON1 = "/rdkp_boom1";
	SlashCmdList["RDKP_BOOMERON"] = RDKP_EnableDebugLogs;
	SLASH_RDKP_BOOMEROFF1 = "/rdkp_boom0";
	SlashCmdList["RDKP_BOOMEROFF"] = RDKP_DisableDebugLogs;
end

-- Based upon our registered events, we want different things to happen when each event occurs.
function RDKP_OnEvent(event)
	if (event == "PLAYER_LOGIN") then
		RDKP_Debug("PLAYER_LOGIN");
		-- Should now have player information
		-- Fired when the player enters the world, enters/leaves an instance, or respawns at a graveyard. Also fires any other time the player sees a loading screen.
		if (RDKP_B_EventFloodBlock == false) then
			RDKP_B_EventFloodBlock = true;
			-- Saved Variables should now be ready.
			-- Any last minute initialization should happen here.
			RDKP_ConvertSettings1xTo2x();
			RDKP_DeployLocalizedStrings();
			RDKP_Khaos_SetupFolder();
			RDKP_Khaos_SetupOptionSet();
			RDKP_AwardPluginInit();
			-- Telepathy registration:
			Telepathy.registerListener(RDKP_SZ_ADDON_PREFIX, "GUILD", RDKP_RecieveInfo_Telepathy, partialCallback)
			Telepathy.registerListener(RDKP_SZ_ADDON_PREFIX, "RAID", RDKP_RecieveInfo_Telepathy, partialCallback)
			Telepathy.registerListener(RDKP_SZ_ADDON_PREFIX, "PARTY", RDKP_RecieveInfo_Telepathy, partialCallback)
			-- Phantom settings showing up despite resetting them constantly:
			if RDKP_Settings then
				if RDKP_Settings.DKP then
					if RDKP_Settings.DKP.Durotar then
						RDKP_Settings.DKP.Durotar = nil;
					end
				end
			end
			if Khaos_Configurations then
				if Khaos_Configurations["configuration"] then
					if Khaos_Configurations["configuration"]["RDKP_OptionSet"] then
						Khaos_Configurations["configuration"]["RDKP_OptionSet"]["RDKP_Settings['DKP']['Durotar']['DEFAULT']"] = nil;
						Khaos_Configurations["configuration"]["RDKP_OptionSet"]["RDKP_Settings['DKP']['Durotar']['Elder Mottled Boar']"] = nil;
					end
				end
			end
			GuildRoster();
			RDKP_SendSettingsUpdateRequest();
			RDKP_Print("Loaded.");
			RDKP_PurgeTransactions();
			RDKP_B_EventFloodBlock = false;
		end
	elseif (event == "CHAT_MSG_LOOT" ) then
		RDKP_Debug("CHAT_MSG_LOOT");
		-- Are we currently recording?
		if (RDKP_Table_RaidInfo["RecordRaid"] == 1) then
			-- When looting, we're going to need to update a given player's GR value.
			-- Need to parse the looting message (code stolen from CTRT)
			RDKP_AwardOnLoot(arg1);
		end
	elseif (event == "CHAT_MSG_WHISPER") then
		RDKP_Debug("CHAT_MSG_WHISPER");
		RDKP_Debug("From: " .. arg2);
		if (RDKP_FilterBy.PST) then
			-- Receive who sent the message.
			-- if they're in the raid, add it to the RDKP_RaidTellList
			if (not RDKP_RaidTellList) then
				RDKP_RaidTellList = {}
			end
			if (RDKP_SearchList(RDKP_Table_RaidPlayers,RDKP_PlayerMap_GetMain(arg2))) then
				tinsert(RDKP_RaidTellList, RDKP_PlayerMap_GetMain(arg2));
				-- And refresh the Priority List.
				RDKP_PopulatePriorityButtons();
			end
			-- Tells not from the Raid, will be be ignored for this purpose.
		end
	elseif (event == "GUILD_ROSTER_UPDATE") then
		RDKP_Debug("GUILD_ROSTER_UPDATE");
		if (RDKP_B_EventFloodBlock == false) then
			RDKP_B_EventFloodBlock = true;
			if (difftime(RDKP_t_GuildRosterUpdateFloodBlock, time()) > (15*60)) then -- Only go through the GuildRosterUpdate work 1ce every 15 mins.
				RDKP_UpdateMasterPlayerList();
				RDKP_PlayerMap_Update();
				RDKP_BroadcastRaids();
				RDKP_t_GuildRosterUpdateFloodBlock = time();
			end
			RDKP_B_EventFloodBlock = false;
		end
	elseif (event == "RAID_ROSTER_UPDATE") then
		RDKP_Debug("RAID_ROSTER_UPDATE");
		--Fired whenever a raid is formed or disbanded, players are leaving or joining a raid (unsure if rejected join requests also fire the event), or when looting rules are changed (regardless of being in raid or party!)
		--* As of 1.12.0, this event no longer fires when entering the world as a raid member. It did previously. Bug? --Mikk 08:54, 28 August 2006 (EDT) 
		RDKP_UpdateRaidList();
		-- Determine if we're in a raid or creating one:
		-- if GetNumRaidMembers() == 0 then
			-- We're leaving raid.
			-- So we should shut down the raid logging here.
			-- RDKP_EndRaid();
			-- return
		if ((not RDKP_Table_RaidInfo["RDKP_bCurrentRaid"]) and (GetNumRaidMembers() > 0)) then
			RDKP_Table_RaidInfo["RDKP_bCurrentRaid"] = true;
		end 
		if (RDKP_Table_RaidInfo["RDKP_bCurrentRaid"] and RDKP_Table_RaidInfo["RecordRaid"]) then
			-- We already are in a raid.
			-- So rules have changed, or roster has changed.
			if (RDKP_Table_RaidInfo["RDKP_RaidID"]) then
				RDKP_Raid_BroadcastKey(RDKP_Table_RaidInfo["RDKP_RaidID"]);
			end
		end
	elseif (event == "PARTY_MEMBERS_CHANGED") then
		-- Fired when the player's party changes.
		-- As of 1.8.3 this event also fires when players are moved around in a Raid and when a player leaves the raid. This holds true even if the changes do not affect your party within the raid.
		-- 4-2-05 Edit: This event is called twice when the event PARTY_LOOT_METHOD_CHANGED is called.
		-- 7-28-05 EDIT: This event is generated whenever someone rejects an invite and your in a group, also generated obviously when someone joins or leaves the group. Also, if for instance you have 3 people in your group and you invite a 4th, it will generate 4 events. If you call GetNumPartyMembers() it will return 0, 1, 2, and 3. First event returing zero, 2nd event returning 1, etc etc. 
		RDKP_UpdateRaidList();
		if (RDKP_Table_RaidInfo["RecordRaid"]) then
			if (RDKP_Table_RaidInfo["RDKP_RaidID"]) then
				RDKP_Raid_BroadcastKey(RDKP_Table_RaidInfo["RDKP_RaidID"]);
			end
		end
	elseif (event == "CHAT_MSG_COMBAT_HOSTILE_DEATH") then
		RDKP_Debug("CHAT_MSG_COMBAT_HOSTILE_DEATH");
		-- This is where per-kill DKP is handled.
		-- arg1 is the string that contains the information we want. 
		-- Get Mob Killed string from arg1
		if (RDKP_Table_RaidInfo["RecordRaid"] == 1) then
			RDKP_AwardOnKill(arg1);
		end
	end
end

function RDKP_Print(...)
	-- Code taken from CTRT's Debug output code w/ only slight modification.
	local sDebug = "Relational DKP: ";
	if (arg) then
		for i = 1, arg.n, 1 do
			if ( arg[i] ) then
				sDebug = sDebug .. arg[i];
			end
		end
	elseif ... then
		sDebug = sDebug .. ...;
	end
	-- MessageFrame:AddMessage("text", r, g, b, a, holdTime) 
	DEFAULT_CHAT_FRAME:AddMessage(sDebug, 1, .25, .75);
end

function RDKP_Debug(...)
	-- Code taken from CTRT's Debug output code w/ only slight modification.
	if ( RDKP_b_Debug ) then
		local sDebug = "# RDKP_DBG: ";
		sDebug = sDebug .. RDKP_TblToStr({...}) .. "#";
		-- MessageFrame:AddMessage("text", r, g, b, a, holdTime) 
		-- DEFAULT_CHAT_FRAME:AddMessage(sDebug, 1, 0.5, 0);
		RDKP_PrintToDebugLog(...);
	end
end

function RDKP_PrintToDebugLog(...)
	if (RDKP_b_Log) then
		if not (RDKP_DebugLog) then
			RDKP_DebugLog = {};
		end
		local szTime = tostring(time());
		if not (RDKP_DebugLog[szTime]) then
			RDKP_DebugLog[szTime] = {};
		end
		tinsert(RDKP_DebugLog[szTime], {...});
	end
end

function RDKP_EnableDebugLogs()
	RDKP_b_Log = 1;
end

function RDKP_DisableDebugLogs()
	RDKP_b_Log = 0;
end

function RDKP_StartRaid()
	RDKP_Debug("RDKP_StartingRaid()" );
	-- This is what will be called when we start a raid.
	-- Are we the raid leader?
	RDKP_SendSettingsUpdateRequest();
	if (IsRaidLeader() or IsPartyLeader() or RDKP_b_Debug) then
		-- Moved the necessary registration for the loot & kill info to only register when we're recording.
		this:RegisterEvent("CHAT_MSG_LOOT");
		this:RegisterEvent("CHAT_MSG_WHISPER");
		this:RegisterEvent("RAID_ROSTER_UPDATE");
		this:RegisterEvent("PARTY_MEMBERS_CHANGED");
		this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
		this:RegisterEvent("CHAT_MSG_ADDON");

		RDKP_Table_RaidInfo["RecordRaid"] = 1;
		RDKP_Table_RaidInfo["RDKP_bCurrentRaid"] = true;
		-- If so create the raid key.
		RDKP_Table_RaidInfo["RDKP_RaidID"] = RDKP_Build_RaidKeyID();
		-- Start awarding DKP based on time.
		-- On the start of the raid, add everyone who's already on the raid list to the timers.
		RDKP_AwardSetupOnTime();
	end
end

function RDKP_EndRaid()
	RDKP_Debug("RDKP_EndRaid()" );
	-- This is what will be called when we end a raid.
	this:UnregisterEvent("CHAT_MSG_LOOT");
	this:UnregisterEvent("CHAT_MSG_WHISPER");
	this:UnregisterEvent("RAID_ROSTER_UPDATE");
	this:UnregisterEvent("PARTY_MEMBERS_CHANGED");
	this:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	this:UnregisterEvent("CHAT_MSG_ADDON");
	RDKP_Table_RaidInfo["RDKP_bCurrentRaid"] = nil;
	RDKP_Table_RaidInfo["RecordRaid"] = 0;
	-- So destroy the local copy of the raid key.
	-- If I rejoin an on-going raid, I will get a key update.
	RDKP_Table_RaidInfo["RDKP_RaidID"] = nil;
	-- Stop awarding DKP based on time.
	Chronos.unscheduleByName("RDKP_DKPforTime");
end

function RDKP_BroadcastRaids()
	RDKP_Debug("RDKP_BroadcastRaids()" );
	-- Take the list of all the raids I know about and broadcast the RaidIDs
	-- We probably want to send the size of the raid as well, so that we can determine if the raids we do have are partials.
	if (RDKP_Table_Raid_Register) then
		for raid, raidvalue in pairs(RDKP_Table_Raid_Register) do
			-- Don't want to broadcast the current raid because it's still in-flux.
			if (RDKP_Table_RaidInfo["RDKP_RaidID"]) then 
				if (not (raid == RDKP_Table_RaidInfo["RDKP_RaidID"])) then
					if (RDKP_Table_Raid_Register[raid].DESTROY) then 
						RDKP_Guild_SendInfo(RDKP_SZ_HAVERAID_PREFIX .. raid .. RDKP_SZ_FIELD_DELIMITER .. "DESTROY");
					else
						local ilength = RDKP_TableGetSize(raidvalue);
						RDKP_Guild_SendInfo(RDKP_SZ_HAVERAID_PREFIX .. raid .. RDKP_SZ_FIELD_DELIMITER .. ilength);
					end
				end
			else
				if (RDKP_Table_Raid_Register[raid].DESTROY) then 
					RDKP_Guild_SendInfo(RDKP_SZ_HAVERAID_PREFIX .. raid .. RDKP_SZ_FIELD_DELIMITER .. "DESTROY");
				else
					local ilength = RDKP_TableGetSize(raidvalue);
					RDKP_Guild_SendInfo(RDKP_SZ_HAVERAID_PREFIX .. raid .. RDKP_SZ_FIELD_DELIMITER .. ilength);
				end
			end
		end
	end
end

-----------------------------------------------------------
-- Communication Functions
-- At some point we will want all clients to be able to have this information.
--
function RDKP_Raid_BroadcastKey(RID)
	if (RID) then
		RDKP_Debug("RDKP_Raid_BroadcastKey(" .. RID .. ")" );
	else
		RDKP_Debug("RDKP_Raid_BroadcastKey()");
	end
	-- Only the RaidLeader is allowed to post the key.
	if ( IsRaidLeader() ) then
		if (RID == nil) then
			RID = RDKP_Build_RaidKeyID();
		end
		local RKID = RDKP_SZ_RAIDKEY_PREFIX .. RID;
		local bResults;
		if RDKP_Table_RaidInfo.RDKP_bCurrentRaid then
			bResults = RDKP_Raid_SendInfo(RKID);
		else
			bResults = RDKP_Party_SendInfo(RKID)
		end
		if ( bResults == nil ) then
			RDKP_Debug("Problem on Broadcast." );
		end
	end
end

function RDKP_Party_SendInfo(szInfo)
	if (szInfo) then 
		RDKP_Debug("RDKP_Party_SendInfo(".. szInfo .. ")" );
	else
		RDKP_Debug("RDKP_Party_SendInfo(nil)" );
		return nil;
	end
	Telepathy.sendMessage(RDKP_SZ_ADDON_PREFIX, szInfo, "PARTY", nil, "NORMAL")
	RDKP_RecieveInfo(szInfo, "PARTY", UnitName("player"));
	return 1;
end

function RDKP_Raid_SendInfo(szInfo)
	if (szInfo) then 
--		RDKP_Debug("RDKP_Raid_SendInfo(".. szInfo .. ")" );
	else
--		RDKP_Debug("RDKP_Raid_SendInfo(nil)" );
		return nil;
	end
	Telepathy.sendMessage(RDKP_SZ_ADDON_PREFIX, szInfo, "RAID", nil, "NORMAL")
	RDKP_RecieveInfo(szInfo, "RAID", UnitName("player"));
	return 1;
end

function RDKP_Guild_SendInfo(szInfo)
	if (szInfo) then 
--		RDKP_Debug("RDKP_Guild_SendInfo(".. szInfo .. ")" );
	else
--		RDKP_Debug("RDKP_Guild_SendInfo(nil)" );
		return nil;
	end
	Telepathy.sendMessage(RDKP_SZ_ADDON_PREFIX, szInfo, "GUILD", nil, "BULK")
	RDKP_RecieveInfo(szInfo, "GUILD", UnitName("player"));
	return 1;
end

function RDKP_RecieveInfo(szInfo, szType, szSource)
	-- RDKP_Debug("RDKP_RecieveInfo(" .. szInfo .. ", " .. szType .. ", " .. szSource .. ")" );
	-- szInfo should be the same as the szInfo that's sent.
	-- Need to parse out the different pieces of information we could receive.

	-- Add the person sending the message to the "connected" list
--	RDKP_AddToConnectedList(szSource);
	-- Setup search pattern string to local 
	local patterns = {
		repRaidKey = "^" .. RDKP_SZ_RAIDKEY_PREFIX .. "(.+)";
		repTransaction = "^" .. RDKP_SZ_TRANSACTION_PREFIX .. "(.+)";
		repTable = "^" .. RDKP_SZ_TABLE .. "(.+)";
		repSettingReq = "^" .. RDKP_SZ_SETTINGSREQUEST .. "(.+)";
		repChar = "^" .. RDKP_SZ_CHAR_TRANSMIT_PREFIX .. "(.+)";
		repHaveRaid = "^" .. RDKP_SZ_HAVERAID_PREFIX .. "(.+)";
		repReqRaid = "^" .. RDKP_SZ_REQUESTRAID_PREFIX .. "(.+)";
		repHash = "^" .. RDKP_SZ_HASH .. "(.+)";
--		repEndRaid = "^" .. RDKP_SZ_ENDRAID_PREFIX .. "(.+)";
	};
	local iStart, iEnd, szCapture, key;
	-- do a search for each of the items above.
	-- Store off the first of what we find.
	for k,v in pairs(patterns) do 
		iStart, iEnd, szCapture= string.find(szInfo, v);
		if iStart then
			key = k; 
			break;
		end
		iStart = nil;
	end
	if (szType == "PARTY" or szType == "RAID") then
		RDKP_Debug("RDKP_RecieveInfo() " .. tostring(szType) );
		if (szType == "RAID") then
			RDKP.tRaidInfo["bCurrentlyInRaid"] = true;
		end
		-- Find if a given pattern is in the string.
		-- If so, then return the data bits of that string.
		-- Check if we're getting a new RaidKeyID:
		if ( key == "repRaidKey" ) then
			if ( not IsRaidLeader() ) then
				RDKP.tRaidInfo["szCurrentRaidID"] = szCapture;
			end
			return true;
		elseif ( key == "repTransaction" ) then
			-- The data received at this point should match a raid that someone (us) are part of.
			-- Process a recieved transaction.
			RDKP_Process_Transaction_Raid(szCapture);
			return true;
		elseif ( key == "repTable" ) then
			-- The data received at this point should match a table that's being broadcast for some reason.
			RDKP_Process_Table_Raid(szCapture, szSource);
			return true;
		end
	elseif (szType == "GUILD") then
		RDKP_Debug("RDKP_RecieveInfo() GUILD" );
		-- For Guild-wide sync:
		-- Settings Request:
		if (key == "repSettingReq") then
			RDKP_ReceiveSettingsUpdateRequest(szCapture);
			return true;
		-- Character Map Broadcast:
		elseif (key == "repChar") then
			RDKP_RecievePlayerMapEntry(szCapture);
			return true;
		-- RaidList Broadcast
		elseif (key == "repHaveRaid") then
			RDKP_RecieveRaidID(szCapture);
			return true;
		-- RaidInfo Request.
		elseif (key == "repReqRaid") then
			RDKP_BroadcastRaidTransactions(szCapture);
			return true;
		-- RaidTransactions 
		elseif (key == "repTransaction") then
			-- The data received at this point should match a historic raid that someone was part of.
			-- Process a recieved transaction.
			RDKP_Process_Transaction_Historic(szCapture);
			return true;
		-- Tables
		elseif (key == "repTable") then
			-- The data received at this point should match a table that's being broadcast for some reason.
			RDKP_Process_Table_Guild(szCapture, szSource);
			return true;
		-- Table hash
		elseif (key == "repHash") then
			-- The data received at this point should match a table that's being broadcast for some reason.
			RDKP_Process_Hash_Guild(szCapture, szSource);
			return true;
		elseif (key == "repEndRaid") then
			-- The data received at this point is the call from the Raid Leader that the Raid is over..
			RDKP_Process_EndRaid(szCapture, szSource);
			return true;
		end
	end
	-- Invalid info there shouldn't be any other channel paths yet.
	local szError = "Invalid Packet From: ".. tostring(szSource) .. "\nTo :" .. tostring(szType) .. "\nData:".. tostring(szInfo);
	RDKP_Debug(szError);
	error(szError);
end

function RDKP_RecieveInfo_Telepathy(message, sender, method)
	-- We're receiving a message via Telepathy managed channels.  
	-- First check to see if it's a table, if so, send off to table handling routines
	-- otherwise send to the old channels if/until they get converted to handle telepathy.
	if type(message) == "table" then
		if (not RDKP_Table_Temp[sender]) then
			RDKP_Table_Temp[sender] = {};
		end
		RDKP_Debug("RDKP_RecieveInfo_Telepathy_TABLE");
		RDKP_Process_Table(RDKP_Table_Temp[sender], message);
	else
		--RDKP_Debug("RDKP_RecieveInfo_Telepathy");
		RDKP_RecieveInfo(message, method, sender);
	end
end

function RDKP_RecieveRaidID(szRaidInfo)
	RDKP_Debug("RDKP_RecieveRaidID(" .. szRaidInfo .. ")" );
	-- And Incoming packet is Stating that someone else has a particular RaidID.
	local szPattern = "(.+)".. RDKP_SZ_FIELD_DELIMITER .."(.+)"
	local iStart, iEnd, szRaidID, iRaidLength = string.find(szRaidInfo, szPattern);	
	RDKP_Debug("RecieveRaidInfo:" .. szRaidID .. ", " .. iRaidLength);
	if (iRaidLength == "DESTROY") then
		RDKP_Table_Raid_Register[szRaidID] = nil;
		RDKP_Table_Raid_Register[szRaidID] = {};
		RDKP_Table_Raid_Register[szRaidID].DESTROY = 1;
	elseif ((not (szRaidID == nil)) and RDKP_Table_Raid_Register[szRaidID] == nil) then
		-- Request that Raid Info.
		RDKP_Table_Raid_Register[szRaidID] = {};
		RDKP_Guild_SendInfo(RDKP_SZ_REQUESTRAID_PREFIX .. szRaidID);
	elseif (RDKP_TableGetSize(RDKP_Table_Raid_Register[szRaidID]) < tonumber(iRaidLength)) then
		-- Request that Raid Info and overwrite what we have.
		RDKP_Guild_SendInfo(RDKP_SZ_REQUESTRAID_PREFIX .. szRaidID);
	end
end

